@@ -1,7 +1,7 @@
 ############################################################################
 #
-# Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-# Contact: http://www.qt-project.org/legal
+# Copyright (C) 2015 The Qt Company Ltd.
+# Contact: http://www.qt.io/licensing
 #
 # This file is part of Qt Creator.
 #
@@ -9,8 +9,8 @@
 # Licensees holding valid commercial Qt licenses may use this file in
 # accordance with the commercial license agreement provided with the
 # Software or, alternatively, in accordance with the terms contained in
-# a written agreement between you and Digia.  For licensing terms and
-# conditions see http://www.qt.io/licensing.  For further information
+# a written agreement between you and The Qt Company.  For licensing terms and
+# conditions see http://www.qt.io/terms-conditions.  For further information
 # use the contact form at http://www.qt.io/contact-us.
 #
 # GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 # requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 # http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 #
-# In addition, as a special exception, Digia gives you certain additional
-# rights.  These rights are described in the Digia Qt LGPL Exception
+# In addition, as a special exception, The Qt Company gives you certain additional
+# rights.  These rights are described in The Qt Company LGPL Exception
 # version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 #
 #############################################################################
@@ -286,7 +286,6 @@ class Dumper(DumperBase):
         self.isShuttingDown_ = False
         self.isInterrupting_ = False
         self.dummyValue = None
-        self.types_ = {}
         self.breakpointsToCheck = set([])
 
     def enterSubItem(self, item):
@@ -369,12 +368,12 @@ class Dumper(DumperBase):
     def callHelper(self, value, func, args):
         # args is a tuple.
         arg = ','.join(args)
-        #warn("CALL: %s -> %s(%s)" % (value, func, arg))
+        #self.warn("CALL: %s -> %s(%s)" % (value, func, arg))
         type = value.type.name
         exp = "((%s*)%s)->%s(%s)" % (type, value.address, func, arg)
-        #warn("CALL: %s" % exp)
+        #self.warn("CALL: %s" % exp)
         result = value.CreateValueFromExpression('', exp)
-        #warn("  -> %s" % result)
+        #self.warn("  -> %s" % result)
         return result
 
     def makeValue(self, type, *args):
@@ -382,9 +381,9 @@ class Dumper(DumperBase):
         frame = thread.GetFrameAtIndex(0)
         inner = ','.join(args)
         value = frame.EvaluateExpression(type + '{' + inner + '}')
-        #warn("  TYPE: %s" % value.type)
-        #warn("  ADDR: 0x%x" % value.address)
-        #warn("  VALUE: %s" % value)
+        #self.warn("  TYPE: %s" % value.type)
+        #self.warn("  ADDR: 0x%x" % value.address)
+        #self.warn("  VALUE: %s" % value)
         return value
 
     def parseAndEvaluate(self, expr):
@@ -551,7 +550,7 @@ class Dumper(DumperBase):
         return self.isKnownMovableType(self.stripNamespaceFromType(type.GetName()))
 
     def putNumChild(self, numchild):
-        #warn("NUM CHILD: '%s' '%s'" % (numchild, self.currentChildNumChild))
+        #self.warn("NUM CHILD: '%s' '%s'" % (numchild, self.currentChildNumChild))
         #if numchild != self.currentChildNumChild:
         self.put('numchild="%s",' % numchild)
 
@@ -611,27 +610,40 @@ class Dumper(DumperBase):
         return re.sub('\\bconst\\b', '', name).replace(' ', '')
 
     def lookupType(self, name):
-        #warn("LOOKUP TYPE NAME: %s" % name)
-        if name.endswith('*'):
-            typeobj = self.lookupType(name[:-1].strip())
-            return typeobj.GetPointerType() if type.IsValid() else None
+        #self.warn("LOOKUP TYPE NAME: %s" % name)
         typeobj = self.target.FindFirstType(name)
-        #warn("LOOKUP RESULT: %s" % typeobj.name)
-        #warn("LOOKUP VALID: %s" % typeobj.IsValid())
         if typeobj.IsValid():
             return typeobj
-        try:
-            if len(self.types_) == 0:
-                for i in xrange(self.target.GetNumModules()):
-                    module = self.target.GetModuleAtIndex(i)
-                    # SBModule.GetType is new somewhere after early 300.x
-                    # So this may fail.
-                    for t in module.GetTypes():
-                        n = self.canonicalTypeName(t.GetName())
-                        self.types_[n] = t
-            return self.types_.get(self.canonicalTypeName(name))
-        except:
-            pass
+        typeobj = self.target.FindFirstType(name + '*')
+        if typeobj.IsValid():
+            return typeob.GetPointeeType()
+        typeobj = self.target.FindFirstType(name + '&')
+        if typeobj.IsValid():
+            return typeob.GetReferencedType()
+        if name.endswith('*'):
+            typeobj = self.target.FindFirstType(name[:-1].strip())
+            if typeobj.IsValid():
+                return typeobj.GetPointerType()
+        #self.warn("LOOKUP RESULT: %s" % typeobj.name)
+        #self.warn("LOOKUP VALID: %s" % typeobj.IsValid())
+        needle = self.canonicalTypeName(name)
+        #self.warn("NEEDLE: %s " % needle)
+        for i in xrange(self.target.GetNumModules()):
+            module = self.target.GetModuleAtIndex(i)
+            # SBModule.GetType is new somewhere after early 300.x
+            # So this may fail.
+            for t in module.GetTypes():
+                n = self.canonicalTypeName(t.GetName())
+                if n == needle:
+                    #self.warn("FOUND TYPE DIRECT 2: %s " % t)
+                    return t
+                if n == needle + '*':
+                    #self.warn("FOUND TYPE BY POINTER 2: %s " % t.GetPointeeType())
+                    return t.GetPointeeType()
+                if n == needle + '&':
+                    #self.warn("FOUND TYPE BY REFERENCE 2: %s " % t)
+                    return t.GetDereferencedType()
+        #self.warn("NOT FOUND: %s " % needle)
         return None
 
     def setupInferior(self, args):
@@ -920,7 +932,6 @@ class Dumper(DumperBase):
         return False
 
     def putItem(self, value, tryDynamic=True):
-        #value = value.GetDynamicValue(lldb.eDynamicCanRunTarget)
         typeName = value.GetType().GetUnqualifiedType().GetName()
         if self.isGoodLldb:
             value.SetPreferDynamicValue(tryDynamic)
@@ -965,10 +976,11 @@ class Dumper(DumperBase):
             realType = value.GetType()
             if hasattr(realType, 'GetCanonicalType'):
                 baseType = realType.GetCanonicalType()
-                baseValue = value.Cast(baseType.unqualified())
-                self.putItem(baseValue)
-                self.putBetterType(realType)
-                return
+                if baseType != realType:
+                    baseValue = value.Cast(baseType.unqualified())
+                    self.putItem(baseValue)
+                    self.putBetterType(realType)
+                    return
 
         # Our turf now.
         if self.isGoodLldb:
@@ -986,11 +998,24 @@ class Dumper(DumperBase):
 
         # References
         if value.GetType().IsReferenceType():
-            origType = value.GetTypeName();
             type = value.GetType().GetDereferencedType().unqualified()
-            addr = int(value) & 0xFFFFFFFFFFFFFFFF
-            self.putItem(value.CreateValueFromAddress(None, addr, type))
-            self.putBetterType(origType)
+            addr = value.GetValueAsUnsigned()
+            #warn("FROM: %s" % value)
+            #warn("ADDR: 0x%x" % addr)
+            #warn("TYPE: %s" % type)
+            # Works:
+            #item = self.currentThread().GetSelectedFrame().EvaluateExpression(
+            #    "(%s*)0x%x" % (type, addr)).Dereference()
+            # Works:
+            item = value.CreateValueFromExpression(None,
+                "(%s*)0x%x" % (type, addr), lldb.SBExpressionOptions()).Dereference()
+            # Does not work:
+            #item = value.CreateValueFromAddress(None, addr, type)
+            # Does not work:
+            #item = value.Cast(type.GetPointerType()).Dereference()
+            #warn("TOOO: %s" % item)
+            self.putItem(item)
+            self.putBetterType(value.GetTypeName())
             return
 
         # Pointers
@@ -1033,7 +1058,7 @@ class Dumper(DumperBase):
                     self.putQObjectGuts(value, staticMetaObject)
 
     def warn(self, msg):
-        self.put('{name="%s",value="",type=""},' % msg)
+        self.put('{name="%s",value="",type="",numchild="0"},' % msg)
 
     def putFields(self, value):
         # Suppress printing of 'name' field for arrays.
