@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,20 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -34,6 +35,7 @@
 #include "qmljsmodelmanagerinterface.h"
 #include "qmljsplugindumper.h"
 #include "qmljstypedescriptionreader.h"
+#include "qmljsdialect.h"
 
 #include <cplusplus/cppmodelmanagerbase.h>
 #include <utils/hostosinfo.h>
@@ -77,6 +79,8 @@ QMLJS_EXPORT Q_LOGGING_CATEGORY(qmljsLog, "qtc.qmljs.common")
 
 static ModelManagerInterface *g_instance = 0;
 
+const char qtQuickUISuffix[] = "ui.qml";
+
 static QStringList environmentImportPaths()
 {
     QStringList paths;
@@ -99,6 +103,7 @@ ModelManagerInterface::ModelManagerInterface(QObject *parent)
       m_defaultProject(0),
       m_pluginDumper(new PluginDumper(this))
 {
+    m_indexerEnabled = qgetenv("QTC_NO_CODE_INDEXER") != "1";
     m_synchronizer.setCancelOnWait(true);
 
     m_updateCppQmlTypesTimer = new QTimer(this);
@@ -113,11 +118,14 @@ ModelManagerInterface::ModelManagerInterface(QObject *parent)
 
     qRegisterMetaType<QmlJS::Document::Ptr>("QmlJS::Document::Ptr");
     qRegisterMetaType<QmlJS::LibraryInfo>("QmlJS::LibraryInfo");
+    qRegisterMetaType<QmlJS::Dialect>("QmlJS::Dialect");
+    qRegisterMetaType<QmlJS::PathAndLanguage>("QmlJS::PathAndLanguage");
+    qRegisterMetaType<QmlJS::PathsAndLanguages>("QmlJS::PathsAndLanguages");
 
-    m_defaultProjectInfo.qtImportsPath = QLibraryInfo::location(QLibraryInfo::ImportsPath);
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-    m_defaultProjectInfo.qtQmlPath = QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath);
-#endif
+    m_defaultProjectInfo.qtImportsPath = QFileInfo(
+                QLibraryInfo::location(QLibraryInfo::ImportsPath)).canonicalFilePath();
+    m_defaultProjectInfo.qtQmlPath = QFileInfo(
+                QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath)).canonicalFilePath();
 
     m_defaultImportPaths << environmentImportPaths();
     updateImportPaths();
@@ -134,39 +142,49 @@ ModelManagerInterface::~ModelManagerInterface()
     g_instance = 0;
 }
 
-static QHash<QString, Language::Enum> defaultLanguageMapping()
+static QHash<QString, Dialect> defaultLanguageMapping()
 {
-    QHash<QString, Language::Enum> res;
-    res[QLatin1String("js")] = Language::JavaScript;
-    res[QLatin1String("qml")] = Language::Qml;
-    res[QLatin1String("qmltypes")] = Language::QmlTypeInfo;
-    res[QLatin1String("qmlproject")] = Language::QmlProject;
-    res[QLatin1String("json")] = Language::Json;
-    res[QLatin1String("qbs")] = Language::QmlQbs;
+    QHash<QString, Dialect> res;
+    res[QLatin1String("js")] = Dialect::JavaScript;
+    res[QLatin1String("qml")] = Dialect::Qml;
+    res[QLatin1String("qmltypes")] = Dialect::QmlTypeInfo;
+    res[QLatin1String("qmlproject")] = Dialect::QmlProject;
+    res[QLatin1String("json")] = Dialect::Json;
+    res[QLatin1String("qbs")] = Dialect::QmlQbs;
+    res[QLatin1String(qtQuickUISuffix)] = Dialect::QmlQtQuick2Ui;
     return res;
 }
 
-Language::Enum ModelManagerInterface::guessLanguageOfFile(const QString &fileName)
+Dialect ModelManagerInterface::guessLanguageOfFile(const QString &fileName)
 {
-    QHash<QString, Language::Enum> lMapping;
+    QHash<QString, Dialect> lMapping;
     if (instance())
         lMapping = instance()->languageForSuffix();
     else
         lMapping = defaultLanguageMapping();
     const QFileInfo info(fileName);
-    const QString fileSuffix = info.suffix();
-    return lMapping.value(fileSuffix, Language::NoLanguage);
+    QString fileSuffix = info.suffix();
+
+    /*
+     * I was reluctant to use complete suffix in all cases, because it is a huge
+     * change in behaivour. But in case of .qml this should be safe.
+     */
+
+    if (fileSuffix == QLatin1String("qml"))
+        fileSuffix = info.completeSuffix();
+
+    return lMapping.value(fileSuffix, Dialect::NoLanguage);
 }
 
-QStringList ModelManagerInterface::globPatternsForLanguages(const QList<Language::Enum> languages)
+QStringList ModelManagerInterface::globPatternsForLanguages(const QList<Dialect> languages)
 {
-    QHash<QString, Language::Enum> lMapping;
+    QHash<QString, Dialect> lMapping;
     if (instance())
         lMapping = instance()->languageForSuffix();
     else
         lMapping = defaultLanguageMapping();
     QStringList patterns;
-    QHashIterator<QString,Language::Enum> i(lMapping);
+    QHashIterator<QString,Dialect> i(lMapping);
     while (i.hasNext()) {
         i.next();
         if (languages.contains(i.value()))
@@ -203,7 +221,7 @@ void ModelManagerInterface::activateScan()
     }
 }
 
-QHash<QString, Language::Enum> ModelManagerInterface::languageForSuffix() const
+QHash<QString, Dialect> ModelManagerInterface::languageForSuffix() const
 {
     return defaultLanguageMapping();
 }
@@ -283,6 +301,8 @@ Snapshot ModelManagerInterface::newestSnapshot() const
 void ModelManagerInterface::updateSourceFiles(const QStringList &files,
                                      bool emitDocumentOnDiskChanged)
 {
+    if (!m_indexerEnabled)
+        return;
     refreshSourceFiles(files, emitDocumentOnDiskChanged);
 }
 
@@ -294,7 +314,7 @@ QFuture<void> ModelManagerInterface::refreshSourceFiles(const QStringList &sourc
 
     QFuture<void> result = QtConcurrent::run(&ModelManagerInterface::parse,
                                               workingCopyInternal(), sourceFiles,
-                                              this, Language::Qml,
+                                              this, Dialect(Dialect::Qml),
                                               emitDocumentOnDiskChanged);
 
     if (m_synchronizer.futures().size() > 10) {
@@ -334,7 +354,7 @@ void ModelManagerInterface::fileChangedOnDisk(const QString &path)
 {
     QtConcurrent::run(&ModelManagerInterface::parse,
                       workingCopyInternal(), QStringList() << path,
-                      this, Language::AnyLanguage, true);
+                      this, Dialect(Dialect::AnyLanguage), true);
 }
 
 void ModelManagerInterface::removeFiles(const QStringList &files)
@@ -394,8 +414,8 @@ bool pInfoLessThanImports(const ModelManagerInterface::ProjectInfo &p1, const Mo
         return true;
     if (p1.qtImportsPath > p2.qtImportsPath)
         return false;
-    QStringList s1 = p1.importPaths;
-    QStringList s2 = p2.importPaths;
+    const PathsAndLanguages &s1 = p1.importPaths;
+    const PathsAndLanguages &s2 = p2.importPaths;
     if (s1.size() < s2.size())
         return true;
     if (s1.size() > s2.size())
@@ -403,7 +423,7 @@ bool pInfoLessThanImports(const ModelManagerInterface::ProjectInfo &p1, const Mo
     for (int i = 0; i < s1.size(); ++i) {
         if (s1.at(i) < s2.at(i))
             return true;
-        else if (s1.at(i) > s2.at(i))
+        else if (s2.at(i) < s1.at(i))
             return false;
     }
     return false;
@@ -501,7 +521,7 @@ ModelManagerInterface::ProjectInfo ModelManagerInterface::projectInfo(
 
 void ModelManagerInterface::updateProjectInfo(const ProjectInfo &pinfo, ProjectExplorer::Project *p)
 {
-    if (! pinfo.isValid() || !p)
+    if (! pinfo.isValid() || !p || !m_indexerEnabled)
         return;
 
     Snapshot snapshot;
@@ -543,9 +563,9 @@ void ModelManagerInterface::updateProjectInfo(const ProjectInfo &pinfo, ProjectE
         if (!snapshot.document(file))
             newFiles += file;
     }
-    updateSourceFiles(newFiles, false);
-    foreach (const QString &newFile, deletedFiles)
+    foreach (const QString &newFile, newFiles)
         m_fileToProject.insert(newFile, p);
+    updateSourceFiles(newFiles, false);
 
     // update qrc cache
     foreach (const QString &newQrc, pinfo.allResourceFiles)
@@ -588,6 +608,10 @@ ModelManagerInterface::ProjectInfo ModelManagerInterface::projectInfoForPath(QSt
     {
         QMutexLocker locker(&m_mutex);
         projects = m_fileToProject.values(path);
+        if (projects.isEmpty()) {
+            QFileInfo fInfo(path);
+            projects = m_fileToProject.values(fInfo.canonicalFilePath());
+        }
     }
     QList<ProjectInfo> infos;
     foreach (ProjectExplorer::Project *project, projects) {
@@ -596,6 +620,7 @@ ModelManagerInterface::ProjectInfo ModelManagerInterface::projectInfoForPath(QSt
             infos.append(info);
     }
     std::sort(infos.begin(), infos.end(), &pInfoLessThanImports);
+    infos.append(m_defaultProjectInfo);
 
     ProjectInfo res;
     foreach (const ProjectInfo &pInfo, infos) {
@@ -603,13 +628,17 @@ ModelManagerInterface::ProjectInfo ModelManagerInterface::projectInfoForPath(QSt
             res.qtImportsPath = pInfo.qtImportsPath;
         if (res.qtQmlPath.isEmpty())
             res.qtQmlPath = pInfo.qtQmlPath;
-        foreach (const QString &path, pInfo.importPaths)
-            if (!res.importPaths.contains(path))
-                res.importPaths.append(path);
+        for (int i = 0; i < pInfo.importPaths.size(); ++i)
+            res.importPaths.maybeInsert(pInfo.importPaths.at(i));
     }
     return res;
 }
 
+bool ModelManagerInterface::isIdle() const
+{
+    return m_synchronizer.futures().isEmpty();
+}
+
 void ModelManagerInterface::emitDocumentChangedOnDisk(Document::Ptr doc)
 { emit documentChangedOnDisk(doc); }
 
@@ -643,7 +672,7 @@ void ModelManagerInterface::updateLibraryInfo(const QString &path, const Library
         emit libraryInfoUpdated(path, info);
 }
 
-static QStringList filesInDirectoryForLanguages(const QString &path, QList<Language::Enum> languages)
+static QStringList filesInDirectoryForLanguages(const QString &path, QList<Dialect> languages)
 {
     const QStringList pattern = ModelManagerInterface::globPatternsForLanguages(languages);
     QStringList files;
@@ -663,7 +692,7 @@ static void findNewImplicitImports(const Document::Ptr &doc, const Snapshot &sna
     if (snapshot.documentsInDirectory(doc->path()).isEmpty()) {
         if (! scannedPaths->contains(doc->path())) {
             *importedFiles += filesInDirectoryForLanguages(doc->path(),
-                                                  Document::companionLanguages(doc->language()));
+                                                  doc->language().companionLanguages());
             scannedPaths->insert(doc->path());
         }
     }
@@ -682,7 +711,7 @@ static void findNewFileImports(const Document::Ptr &doc, const Snapshot &snapsho
             if (snapshot.documentsInDirectory(importName).isEmpty()) {
                 if (! scannedPaths->contains(importName)) {
                     *importedFiles += filesInDirectoryForLanguages(importName,
-                                                          Document::companionLanguages(doc->language()));
+                                                          doc->language().companionLanguages());
                     scannedPaths->insert(importName);
                 }
             }
@@ -696,7 +725,7 @@ static void findNewFileImports(const Document::Ptr &doc, const Snapshot &snapsho
             QMapIterator<QString,QStringList> dirContents(ModelManagerInterface::instance()->filesInQrcPath(importName));
             while (dirContents.hasNext()) {
                 dirContents.next();
-                if (Document::isQmlLikeOrJsLanguage(ModelManagerInterface::guessLanguageOfFile(dirContents.key()))) {
+                if (ModelManagerInterface::guessLanguageOfFile(dirContents.key()).isQmlLikeOrJsLanguage()) {
                     foreach (const QString &filePath, dirContents.value()) {
                         if (! snapshot.document(filePath))
                             *importedFiles += filePath;
@@ -759,7 +788,7 @@ static bool findNewQmlLibraryInPath(const QString &path,
             const QString path = QDir::cleanPath(componentFileInfo.absolutePath());
             if (! scannedPaths->contains(path)) {
                 *importedFiles += filesInDirectoryForLanguages(path,
-                                                      Document::companionLanguages(Language::AnyLanguage));
+                     Dialect(Dialect::AnyLanguage).companionLanguages());
                 scannedPaths->insert(path);
             }
         }
@@ -806,7 +835,7 @@ static void findNewLibraryImports(const Document::Ptr &doc, const Snapshot &snap
                             importedFiles, scannedPaths, newLibraries, false);
 
     // scan dir and lib imports
-    const QStringList importPaths = modelManager->importPaths();
+    const PathsAndLanguages importPaths = modelManager->importPaths();
     foreach (const ImportInfo &import, doc->bind()->imports()) {
         if (import.type() == ImportType::Directory) {
             const QString targetPath = import.path();
@@ -817,8 +846,8 @@ static void findNewLibraryImports(const Document::Ptr &doc, const Snapshot &snap
         if (import.type() == ImportType::Library) {
             if (!import.version().isValid())
                 continue;
-            foreach (const QString &importPath, importPaths) {
-                const QString targetPath = QDir(importPath).filePath(import.path());
+            foreach (const PathAndLanguage &importPath, importPaths) {
+                const QString targetPath = importPath.path().appendPath(import.path()).toString();
                 findNewQmlLibrary(targetPath, import.version(), snapshot, modelManager,
                                   importedFiles, scannedPaths, newLibraries);
             }
@@ -831,7 +860,7 @@ void ModelManagerInterface::parseLoop(QSet<QString> &scannedPaths,
                              WorkingCopy workingCopy,
                              QStringList files,
                              ModelManagerInterface *modelManager,
-                             Language::Enum mainLanguage,
+                             Dialect mainLanguage,
                              bool emitDocChangedOnDisk,
                              std::function<bool(qreal)> reportProgress)
 {
@@ -841,15 +870,19 @@ void ModelManagerInterface::parseLoop(QSet<QString> &scannedPaths,
 
         const QString fileName = files.at(i);
 
-        Language::Enum language = guessLanguageOfFile(fileName);
-        if (language == Language::NoLanguage) {
+        Dialect language = guessLanguageOfFile(fileName);
+        if (language == Dialect::NoLanguage) {
             if (fileName.endsWith(QLatin1String(".qrc")))
                 modelManager->updateQrcFile(fileName);
             continue;
         }
-        if (language == Language::Qml
-                && (mainLanguage == Language::QmlQtQuick1 || mainLanguage == Language::QmlQtQuick2))
+        if (language == Dialect::Qml
+                && (mainLanguage == Dialect::QmlQtQuick1 || mainLanguage == Dialect::QmlQtQuick2))
             language = mainLanguage;
+        if (language == Dialect::Qml && mainLanguage == Dialect::QmlQtQuick2Ui)
+            language = Dialect::QmlQtQuick2;
+        if (language == Dialect::QmlTypeInfo || language == Dialect::QmlProject)
+            continue;
         QString contents;
         int documentRevision = 0;
 
@@ -917,7 +950,7 @@ void ModelManagerInterface::parse(QFutureInterface<void> &future,
                          WorkingCopy workingCopy,
                          QStringList files,
                          ModelManagerInterface *modelManager,
-                         Language::Enum mainLanguage,
+                         Dialect mainLanguage,
                          bool emitDocChangedOnDisk)
 {
     FutureReporter reporter(future);
@@ -935,19 +968,23 @@ void ModelManagerInterface::parse(QFutureInterface<void> &future,
 struct ScanItem {
     QString path;
     int depth;
-    ScanItem(QString path = QString(), int depth = 0)
-        : path(path), depth(depth)
+    Dialect language;
+    ScanItem(QString path = QString(), int depth = 0, Dialect language = Dialect::AnyLanguage)
+        : path(path), depth(depth), language(language)
     { }
 };
 
 void ModelManagerInterface::importScan(QFutureInterface<void> &future,
                               ModelManagerInterface::WorkingCopy workingCopy,
-                              QStringList paths, ModelManagerInterface *modelManager,
-                              Language::Enum language,
+                              PathsAndLanguages paths, ModelManagerInterface *modelManager,
                               bool emitDocChangedOnDisk)
 {
     // paths we have scanned for files and added to the files list
-    QSet<QString> scannedPaths = modelManager->m_scannedPaths;
+    QSet<QString> scannedPaths;
+    {
+        QMutexLocker l(&modelManager->m_mutex);
+        scannedPaths = modelManager->m_scannedPaths;
+    }
     // libraries we've found while scanning imports
     QSet<QString> newLibraries;
 
@@ -955,11 +992,12 @@ void ModelManagerInterface::importScan(QFutureInterface<void> &future,
     pathsToScan.reserve(paths.size());
     {
         QMutexLocker l(&modelManager->m_mutex);
-        foreach (const QString &path, paths) {
-            QString cPath = QDir::cleanPath(path);
+        for (int i = 0; i < paths.size(); ++i) {
+            PathAndLanguage pAndL = paths.at(i);
+            QString cPath = QDir::cleanPath(pAndL.path().toString());
             if (modelManager->m_scannedPaths.contains(cPath))
                 continue;
-            pathsToScan.append(ScanItem(cPath));
+            pathsToScan.append(ScanItem(cPath, 0, pAndL.language()));
             modelManager->m_scannedPaths.insert(cPath);
         }
     }
@@ -969,7 +1007,8 @@ void ModelManagerInterface::importScan(QFutureInterface<void> &future,
     future.setProgressRange(0, progressRange); // update max length while iterating?
     const bool libOnly = true; // FIXME remove when tested more
     const Snapshot snapshot = modelManager->snapshot();
-    while (!pathsToScan.isEmpty() && !future.isCanceled()) {
+    bool isCanceled = future.isCanceled();
+    while (!pathsToScan.isEmpty() && !isCanceled) {
         ScanItem toScan = pathsToScan.last();
         pathsToScan.pop_back();
         int pathBudget = (1 << (maxScanDepth + 2 - toScan.depth));
@@ -979,14 +1018,14 @@ void ModelManagerInterface::importScan(QFutureInterface<void> &future,
                                          &scannedPaths, &newLibraries, true)
                     && !libOnly && snapshot.documentsInDirectory(toScan.path).isEmpty())
                 importedFiles += filesInDirectoryForLanguages(toScan.path,
-                                                     Document::companionLanguages(language));
+                                                     toScan.language.companionLanguages());
             workDone += 1;
             future.setProgressValue(progressRange * workDone / totalWork);
             if (!importedFiles.isEmpty()) {
                 FutureReporter reporter(future, progressRange * pathBudget / (4 * totalWork),
                                         progressRange * workDone / totalWork);
                 parseLoop(scannedPaths, newLibraries, workingCopy, importedFiles, modelManager,
-                          language, emitDocChangedOnDisk, reporter); // run in parallel??
+                          toScan.language, emitDocChangedOnDisk, reporter); // run in parallel??
                 importedFiles.clear();
             }
             workDone += pathBudget / 4 - 1;
@@ -1001,22 +1040,23 @@ void ModelManagerInterface::importScan(QFutureInterface<void> &future,
             workDone += 1;
             totalWork += pathBudget / 2 * subDirs.size() - pathBudget * 3 / 4 + 1;
             foreach (const QString path, subDirs)
-                pathsToScan.append(ScanItem(dir.absoluteFilePath(path), toScan.depth + 1));
+                pathsToScan.append(ScanItem(dir.absoluteFilePath(path), toScan.depth + 1, toScan.language));
         } else {
             workDone += pathBudget * 3 / 4;
         }
         future.setProgressValue(progressRange * workDone / totalWork);
+        isCanceled = future.isCanceled();
     }
     future.setProgressValue(progressRange);
-    if (future.isCanceled()) {
+    if (isCanceled) {
         // assume no work has been done
         QMutexLocker l(&modelManager->m_mutex);
-        foreach (const QString &path, paths)
-            modelManager->m_scannedPaths.remove(path);
+        for (int i = 0; i < paths.size(); ++i)
+            modelManager->m_scannedPaths.remove(paths.at(i).path().toString());
     }
 }
 
-QStringList ModelManagerInterface::importPaths() const
+PathsAndLanguages ModelManagerInterface::importPaths() const
 {
     QMutexLocker l(&m_mutex);
     return m_allImportPaths;
@@ -1034,23 +1074,22 @@ QmlLanguageBundles ModelManagerInterface::extendedBundles() const
     return m_extendedBundles;
 }
 
-void ModelManagerInterface::maybeScan(const QStringList &importPaths,
-                                      Language::Enum defaultLanguage)
+void ModelManagerInterface::maybeScan(const PathsAndLanguages &importPaths)
 {
-    QStringList pathToScan;
+    if (!m_indexerEnabled)
+        return;
+    PathsAndLanguages pathToScan;
     {
         QMutexLocker l(&m_mutex);
-        foreach (QString importPath, importPaths)
-            if (!m_scannedPaths.contains(importPath)) {
-                pathToScan.append(importPath);
-            }
+        foreach (const PathAndLanguage &importPath, importPaths)
+            if (!m_scannedPaths.contains(importPath.path().toString()))
+                pathToScan.maybeInsert(importPath);
     }
 
-    if (pathToScan.count() > 1) {
+    if (pathToScan.length() > 1) {
         QFuture<void> result = QtConcurrent::run(&ModelManagerInterface::importScan,
                                                   workingCopyInternal(), pathToScan,
-                                                  this, defaultLanguage,
-                                                  true);
+                                                  this, true);
 
         if (m_synchronizer.futures().size() > 10) {
             QList<QFuture<void> > futures = m_synchronizer.futures();
@@ -1071,33 +1110,39 @@ void ModelManagerInterface::maybeScan(const QStringList &importPaths,
 
 void ModelManagerInterface::updateImportPaths()
 {
-    QStringList allImportPaths;
+    if (!m_indexerEnabled)
+        return;
+    PathsAndLanguages allImportPaths;
     QmlLanguageBundles activeBundles;
     QmlLanguageBundles extendedBundles;
     QMapIterator<ProjectExplorer::Project *, ProjectInfo> pInfoIter(m_projects);
-    QHashIterator<Language::Enum, QmlJS::ViewerContext> vCtxsIter = m_defaultVContexts;
+    QHashIterator<Dialect, QmlJS::ViewerContext> vCtxsIter = m_defaultVContexts;
     while (pInfoIter.hasNext()) {
         pInfoIter.next();
-        foreach (const QString &path, pInfoIter.value().importPaths) {
-            const QString canonicalPath = QFileInfo(path).canonicalFilePath();
+        const PathsAndLanguages &iPaths = pInfoIter.value().importPaths;
+        for (int i = 0; i < iPaths.size(); ++i) {
+            PathAndLanguage pAndL = iPaths.at(i);
+            const QString canonicalPath = pAndL.path().toFileInfo().canonicalFilePath();
             if (!canonicalPath.isEmpty())
-                allImportPaths += canonicalPath;
+                allImportPaths.maybeInsert(Utils::FileName::fromString(canonicalPath),
+                                           pAndL.language());
         }
     }
     while (vCtxsIter.hasNext()) {
         vCtxsIter.next();
-        allImportPaths << vCtxsIter.value().paths;
+        foreach (const QString &path, vCtxsIter.value().paths)
+            allImportPaths.maybeInsert(Utils::FileName::fromString(path), vCtxsIter.value().language);
     }
     pInfoIter.toFront();
     while (pInfoIter.hasNext()) {
         pInfoIter.next();
         activeBundles.mergeLanguageBundles(pInfoIter.value().activeBundle);
-        foreach (Language::Enum l, pInfoIter.value().activeBundle.languages()) {
+        foreach (Dialect l, pInfoIter.value().activeBundle.languages()) {
             foreach (const QString &path, pInfoIter.value().activeBundle.bundleForLanguage(l)
                  .searchPaths().stringList()) {
                 const QString canonicalPath = QFileInfo(path).canonicalFilePath();
                 if (!canonicalPath.isEmpty())
-                    allImportPaths += canonicalPath;
+                    allImportPaths.maybeInsert(Utils::FileName::fromString(canonicalPath), l);
             }
         }
     }
@@ -1105,28 +1150,29 @@ void ModelManagerInterface::updateImportPaths()
     while (pInfoIter.hasNext()) {
         pInfoIter.next();
         QString pathAtt = pInfoIter.value().qtQmlPath;
-        if (!pathAtt.isEmpty() && (allImportPaths.isEmpty() || allImportPaths.last() != pathAtt))
-            allImportPaths.append(pathAtt);
+        if (!pathAtt.isEmpty())
+            allImportPaths.maybeInsert(Utils::FileName::fromString(pathAtt), Dialect::QmlQtQuick2);
     }
     {
         QString pathAtt = defaultProjectInfo().qtQmlPath;
-        if (!pathAtt.isEmpty() && (allImportPaths.isEmpty() || allImportPaths.last() != pathAtt))
-            allImportPaths.append(pathAtt);
+        if (!pathAtt.isEmpty())
+            allImportPaths.maybeInsert(Utils::FileName::fromString(pathAtt), Dialect::QmlQtQuick2);
     }
     pInfoIter.toFront();
     while (pInfoIter.hasNext()) {
         pInfoIter.next();
         QString pathAtt = pInfoIter.value().qtImportsPath;
-        if (!pathAtt.isEmpty() && (allImportPaths.isEmpty() || allImportPaths.last() != pathAtt))
-            allImportPaths.append(pathAtt);
+        if (!pathAtt.isEmpty())
+            allImportPaths.maybeInsert(Utils::FileName::fromString(pathAtt), Dialect::QmlQtQuick1);
     }
     {
         QString pathAtt = defaultProjectInfo().qtImportsPath;
-        if (!pathAtt.isEmpty() && (allImportPaths.isEmpty() || allImportPaths.last() != pathAtt))
-            allImportPaths.append(pathAtt);
+        if (!pathAtt.isEmpty())
+            allImportPaths.maybeInsert(Utils::FileName::fromString(pathAtt), Dialect::QmlQtQuick1);
     }
-    allImportPaths += m_defaultImportPaths;
-    allImportPaths.removeDuplicates();
+    foreach (const QString &path, m_defaultImportPaths)
+        allImportPaths.maybeInsert(Utils::FileName::fromString(path), Dialect::Qml);
+    allImportPaths.compact();
 
     {
         QMutexLocker l(&m_mutex);
@@ -1148,7 +1194,7 @@ void ModelManagerInterface::updateImportPaths()
 
     if (!m_shouldScanImports)
         return;
-    maybeScan(allImportPaths, Language::Qml);
+    maybeScan(allImportPaths);
 }
 
 void ModelManagerInterface::loadPluginTypes(const QString &libraryPath, const QString &importPath,
@@ -1282,9 +1328,11 @@ ModelManagerInterface::CppDataHash ModelManagerInterface::cppData() const
 
 LibraryInfo ModelManagerInterface::builtins(const Document::Ptr &doc) const
 {
-    ProjectInfo info = projectInfoForPath(doc->path());
+    ProjectInfo info = projectInfoForPath(doc->fileName());
     if (!info.isValid())
         return LibraryInfo();
+    if (!info.qtQmlPath.isEmpty())
+        return m_validSnapshot.libraryInfo(info.qtQmlPath);
     return m_validSnapshot.libraryInfo(info.qtImportsPath);
 }
 
@@ -1294,14 +1342,15 @@ ViewerContext ModelManagerInterface::completeVContext(const ViewerContext &vCtx,
     ViewerContext res = vCtx;
 
     if (!doc.isNull()
-            && ((vCtx.language == Language::AnyLanguage && doc->language() != Language::NoLanguage)
-                || (vCtx.language == Language::Qml
-                    && (doc->language() == Language::QmlQtQuick1
-                        || doc->language() == Language::QmlQtQuick2))))
+            && ((vCtx.language == Dialect::AnyLanguage && doc->language() != Dialect::NoLanguage)
+                || (vCtx.language == Dialect::Qml
+                    && (doc->language() == Dialect::QmlQtQuick1
+                        || doc->language() == Dialect::QmlQtQuick2
+                        || doc->language() == Dialect::QmlQtQuick2Ui))))
         res.language = doc->language();
     ProjectInfo info;
     if (!doc.isNull())
-        info = projectInfoForPath(doc->path());
+        info = projectInfoForPath(doc->fileName());
     ViewerContext defaultVCtx = defaultVContext(res.language, Document::Ptr(0), false);
     ProjectInfo defaultInfo = defaultProjectInfo();
     if (info.qtImportsPath.isEmpty())
@@ -1318,17 +1367,18 @@ ViewerContext ModelManagerInterface::completeVContext(const ViewerContext &vCtx,
     {
         foreach (const QString &path, defaultVCtx.paths)
             res.maybeAddPath(path);
-        switch (res.language) {
-        case Language::AnyLanguage:
-        case Language::Qml:
+        switch (res.language.dialect()) {
+        case Dialect::AnyLanguage:
+        case Dialect::Qml:
             res.maybeAddPath(info.qtQmlPath);
             // fallthrough
-        case Language::QmlQtQuick1:
+        case Dialect::QmlQtQuick1:
             res.maybeAddPath(info.qtImportsPath);
             // fallthrough
-        case Language::QmlQtQuick2:
+        case Dialect::QmlQtQuick2:
+        case Dialect::QmlQtQuick2Ui:
         {
-            if (res.language == Language::QmlQtQuick2)
+            if (res.language == Dialect::QmlQtQuick2 || res.language == Dialect::QmlQtQuick2Ui)
                 res.maybeAddPath(info.qtQmlPath);
             QList<ProjectInfo> allProjects;
             {
@@ -1336,18 +1386,24 @@ ViewerContext ModelManagerInterface::completeVContext(const ViewerContext &vCtx,
                 allProjects = m_projects.values();
             }
             std::sort(allProjects.begin(), allProjects.end(), &pInfoLessThanImports);
+            QList<Dialect> languages = res.language.companionLanguages();
             foreach (const ProjectInfo &pInfo, allProjects) {
-                foreach (const QString &path, pInfo.importPaths)
-                    res.maybeAddPath(path);
+                for (int i = 0; i< pInfo.importPaths.size(); ++i) {
+                    PathAndLanguage pAndL = pInfo.importPaths.at(i);
+                    if (languages.contains(pAndL.language()) || pAndL.language().companionLanguages().contains(res.language))
+                        res.maybeAddPath(pAndL.path().toString());
+                }
             }
+            foreach (const QString &path, environmentImportPaths())
+                res.maybeAddPath(path);
             break;
         }
-        case Language::NoLanguage:
-        case Language::JavaScript:
-        case Language::QmlTypeInfo:
-        case Language::Json:
-        case Language::QmlQbs:
-        case Language::QmlProject:
+        case Dialect::NoLanguage:
+        case Dialect::JavaScript:
+        case Dialect::QmlTypeInfo:
+        case Dialect::Json:
+        case Dialect::QmlQbs:
+        case Dialect::QmlProject:
             break;
         }
         break;
@@ -1358,27 +1414,34 @@ ViewerContext ModelManagerInterface::completeVContext(const ViewerContext &vCtx,
     case ViewerContext::AddDefaultPaths:
         foreach (const QString &path, defaultVCtx.paths)
             res.maybeAddPath(path);
-        if (res.language == Language::AnyLanguage || res.language == Language::Qml
-                || res.language == Language::QmlQtQuick2)
+        if (res.language == Dialect::AnyLanguage || res.language == Dialect::Qml
+                || res.language == Dialect::QmlQtQuick2 || res.language == Dialect::QmlQtQuick2Ui)
             res.maybeAddPath(info.qtImportsPath);
-        if (res.language == Language::AnyLanguage || res.language == Language::Qml
-                || res.language == Language::QmlQtQuick1)
+        if (res.language == Dialect::AnyLanguage || res.language == Dialect::Qml
+                || res.language == Dialect::QmlQtQuick1)
             res.maybeAddPath(info.qtQmlPath);
+        if (res.language == Dialect::AnyLanguage || res.language == Dialect::Qml
+                || res.language == Dialect::QmlQtQuick1 || res.language == Dialect::QmlQtQuick2
+                || res.language == Dialect::QmlQtQuick2Ui) {
+            foreach (const QString &path, environmentImportPaths())
+                res.maybeAddPath(path);
+        }
         break;
     }
     res.flags = ViewerContext::Complete;
     return res;
 }
 
-ViewerContext ModelManagerInterface::defaultVContext(Language::Enum language,
+ViewerContext ModelManagerInterface::defaultVContext(Dialect language,
                                                      const Document::Ptr &doc,
                                                      bool autoComplete) const
 {
     if (!doc.isNull()) {
-        if (language == Language::AnyLanguage && doc->language() != Language::NoLanguage)
+        if (language == Dialect::AnyLanguage && doc->language() != Dialect::NoLanguage)
             language = doc->language();
-        else if (language == Language::Qml &&
-                 (doc->language() == Language::QmlQtQuick1 || doc->language() == Language::QmlQtQuick2))
+        else if (language == Dialect::Qml &&
+                 (doc->language() == Dialect::QmlQtQuick1 || doc->language() == Dialect::QmlQtQuick2
+                  || doc->language() == Dialect::QmlQtQuick2Ui))
             language = doc->language();
     }
     ViewerContext defaultCtx;
@@ -1399,6 +1462,12 @@ ModelManagerInterface::ProjectInfo ModelManagerInterface::defaultProjectInfo() c
     return m_defaultProjectInfo;
 }
 
+ModelManagerInterface::ProjectInfo ModelManagerInterface::defaultProjectInfoForProject(
+        ProjectExplorer::Project *) const
+{
+    return ModelManagerInterface::ProjectInfo();
+}
+
 void ModelManagerInterface::setDefaultVContext(const ViewerContext &vContext)
 {
     QMutexLocker locker(&m_mutex);
@@ -1415,7 +1484,7 @@ Document::Ptr ModelManagerInterface::ensuredGetDocumentForPath(const QString &fi
 {
     QmlJS::Document::Ptr document = newestSnapshot().document(filePath);
     if (!document) {
-        document = QmlJS::Document::create(filePath, QmlJS::Language::Qml);
+        document = QmlJS::Document::create(filePath, QmlJS::Dialect::Qml);
         QMutexLocker lock(&m_mutex);
 
         m_newestSnapshot.insert(document);
