@@ -1,7 +1,7 @@
 /**************************************************************************
 **
-** Copyright (c) 2014 BogDan Vatra <bog_dan_ro@yahoo.com>
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 BogDan Vatra <bog_dan_ro@yahoo.com>
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -74,7 +74,7 @@ using namespace ProjectExplorer;
 using namespace Utils;
 
 namespace Android {
-namespace Internal {
+using namespace Internal;
 
 namespace {
     const QLatin1String SettingsGroup("AndroidConfigurations");
@@ -85,6 +85,7 @@ namespace {
     const QLatin1String OpenJDKLocationKey("OpenJDKLocation");
     const QLatin1String KeystoreLocationKey("KeystoreLocation");
     const QLatin1String AutomaticKitCreationKey("AutomatiKitCreation");
+    const QLatin1String UseGradleKey("UseGradle");
     const QLatin1String MakeExtraSearchDirectory("MakeExtraSearchDirectory");
     const QLatin1String DefaultDevice("DefaultDevice");
     const QLatin1String PartitionSizeKey("PartitionSize");
@@ -108,7 +109,7 @@ namespace {
 
     bool androidDevicesLessThan(const AndroidDeviceInfo &dev1, const AndroidDeviceInfo &dev2)
     {
-        if (dev1.serialNumber.contains(QLatin1String("????")) == dev2.serialNumber.contains(QLatin1String("????")))
+        if (dev1.serialNumber.contains(QLatin1String("????")) != dev2.serialNumber.contains(QLatin1String("????")))
             return !dev1.serialNumber.contains(QLatin1String("????"));
         if (dev1.type != dev2.type)
             return dev1.type == AndroidDeviceInfo::Hardware;
@@ -130,6 +131,33 @@ namespace {
         }
         return res;
     }
+
+    static bool is32BitUserSpace()
+    {
+        // Do the exact same check as android's emulator is doing:
+        if (Utils::HostOsInfo::isLinuxHost()) {
+            if (QSysInfo::WordSize == 32 ) {
+                Utils::Environment env = Utils::Environment::systemEnvironment();
+                QString executable = env.searchInPath(QLatin1String("file")).toString();
+                QString shell = env.value(QLatin1String("SHELL"));
+                if (executable.isEmpty() || shell.isEmpty())
+                    return true; // we can't detect, but creator is 32bit so assume 32bit
+
+                QProcess proc;
+                proc.setProcessChannelMode(QProcess::MergedChannels);
+                proc.start(executable, QStringList() << shell);
+                if (!proc.waitForFinished(2000)) {
+                    proc.kill();
+                    return true;
+                }
+                if (proc.readAll().contains("x86-64"))
+                    return false;
+                return true;
+            }
+        }
+        return false;
+    }
+
 }
 
 //////////////////////////////////
@@ -182,6 +210,7 @@ void AndroidConfig::load(const QSettings &settings)
     m_sdkLocation = FileName::fromString(settings.value(SDKLocationKey).toString());
     m_ndkLocation = FileName::fromString(settings.value(NDKLocationKey).toString());
     m_antLocation = FileName::fromString(settings.value(AntLocationKey).toString());
+    m_useGradle = settings.value(UseGradleKey, false).toBool();
     m_openJDKLocation = FileName::fromString(settings.value(OpenJDKLocationKey).toString());
     m_keystoreLocation = FileName::fromString(settings.value(KeystoreLocationKey).toString());
     m_toolchainHost = settings.value(ToolchainHostKey).toString();
@@ -215,7 +244,8 @@ void AndroidConfig::load(const QSettings &settings)
 }
 
 AndroidConfig::AndroidConfig()
-    : m_availableSdkPlatformsUpToDate(false),
+    : m_useGradle(false),
+      m_availableSdkPlatformsUpToDate(false),
       m_NdkInformationUpToDate(false)
 {
 
@@ -231,6 +261,7 @@ void AndroidConfig::save(QSettings &settings) const
     settings.setValue(SDKLocationKey, m_sdkLocation.toString());
     settings.setValue(NDKLocationKey, m_ndkLocation.toString());
     settings.setValue(AntLocationKey, m_antLocation.toString());
+    settings.setValue(UseGradleKey, m_useGradle);
     settings.setValue(OpenJDKLocationKey, m_openJDKLocation.toString());
     settings.setValue(KeystoreLocationKey, m_keystoreLocation.toString());
     settings.setValue(PartitionSizeKey, m_partitionSize);
@@ -338,10 +369,12 @@ void AndroidConfig::updateAvailableSdkPlatforms() const
 
 QStringList AndroidConfig::apiLevelNamesFor(const QList<SdkPlatform> &platforms)
 {
-    QStringList results;
-    foreach (const SdkPlatform &platform, platforms)
-        results << QLatin1String("android-") + QString::number(platform.apiLevel);
-    return results;
+    return Utils::transform(platforms, AndroidConfig::apiLevelNameFor);
+}
+
+QString AndroidConfig::apiLevelNameFor(const SdkPlatform &platform)
+{
+    return QLatin1String("android-") + QString::number(platform.apiLevel);
 }
 
 QList<SdkPlatform> AndroidConfig::sdkTargets(int minApiLevel) const
@@ -378,7 +411,7 @@ FileName AndroidConfig::androidToolPath() const
         // Java and I've made some progress on it. So if android.exe exists, return that instead.
         FileName path = m_sdkLocation;
         path.appendPath(QLatin1String("tools/android" QTC_HOST_EXE_SUFFIX));
-        if (path.toFileInfo().exists())
+        if (path.exists())
             return path;
         path = m_sdkLocation;
         return path.appendPath(QLatin1String("tools/android" ANDROID_BAT_SUFFIX));
@@ -412,16 +445,6 @@ FileName AndroidConfig::toolPath(Abi::Architecture architecture, const QString &
             .arg(toolsPrefix(architecture)));
 }
 
-FileName AndroidConfig::stripPath(Abi::Architecture architecture, const QString &ndkToolChainVersion) const
-{
-    return toolPath(architecture, ndkToolChainVersion).appendString(QLatin1String("-strip" QTC_HOST_EXE_SUFFIX));
-}
-
-FileName AndroidConfig::readelfPath(Abi::Architecture architecture, const QString &ndkToolChainVersion) const
-{
-    return toolPath(architecture, ndkToolChainVersion).appendString(QLatin1String("-readelf" QTC_HOST_EXE_SUFFIX));
-}
-
 FileName AndroidConfig::gccPath(Abi::Architecture architecture, const QString &ndkToolChainVersion) const
 {
     return toolPath(architecture, ndkToolChainVersion).appendString(QLatin1String("-gcc" QTC_HOST_EXE_SUFFIX));
@@ -445,17 +468,6 @@ FileName AndroidConfig::keytoolPath() const
     return openJDKBinPath().appendPath(keytoolName);
 }
 
-FileName AndroidConfig::jarsignerPath() const
-{
-    return openJDKBinPath().appendPath(jarsignerName);
-}
-
-FileName AndroidConfig::zipalignPath() const
-{
-    FileName path = m_sdkLocation;
-    return path.appendPath(QLatin1String("tools/zipalign" QTC_HOST_EXE_SUFFIX));
-}
-
 QVector<AndroidDeviceInfo> AndroidConfig::connectedDevices(QString *error) const
 {
     QVector<AndroidDeviceInfo> devices;
@@ -538,7 +550,7 @@ AndroidConfig::CreateAvdInfo AndroidConfig::createAVDImpl(CreateAvdInfo info, Ut
     proc.start(androidToolPath.toString(), arguments);
     if (!proc.waitForStarted()) {
         info.error = QApplication::translate("AndroidConfig", "Could not start process \"%1 %2\"")
-                .arg(androidToolPath.toString(), arguments.join(QLatin1String(" ")));
+                .arg(androidToolPath.toString(), arguments.join(QLatin1Char(' ')));
         return info;
     }
 
@@ -675,9 +687,13 @@ bool AndroidConfig::startAVDAsync(const QString &avdName) const
     avdProcess->connect(avdProcess, SIGNAL(finished(int)), avdProcess, SLOT(deleteLater()));
 
     // start the emulator
-    avdProcess->start(emulatorToolPath().toString(),
-                        QStringList() << QLatin1String("-partition-size") << QString::number(partitionSize())
-                        << QLatin1String("-avd") << avdName);
+    QStringList arguments;
+    if (AndroidConfigurations::force32bitEmulator())
+        arguments << QLatin1String("-force-32bit");
+
+    arguments << QLatin1String("-partition-size") << QString::number(partitionSize())
+              << QLatin1String("-avd") << avdName;
+    avdProcess->start(emulatorToolPath().toString(), arguments);
     if (!avdProcess->waitForStarted(-1)) {
         delete avdProcess;
         return false;
@@ -824,6 +840,24 @@ bool AndroidConfig::hasFinishedBooting(const QString &device) const
 QStringList AndroidConfig::getAbis(const QString &device) const
 {
     QStringList result;
+    // First try via ro.product.cpu.abilist
+    QStringList arguments = AndroidDeviceInfo::adbSelector(device);
+    arguments << QLatin1String("shell") << QLatin1String("getprop");
+    arguments << QLatin1String("ro.product.cpu.abilist");
+    QProcess adbProc;
+    adbProc.start(adbToolPath().toString(), arguments);
+    if (!adbProc.waitForFinished(10000)) {
+        adbProc.kill();
+        return result;
+    }
+    QString output = QString::fromLocal8Bit(adbProc.readAll().trimmed());
+    if (!output.isEmpty()) {
+        QStringList result = output.split(QLatin1Char(','));
+        if (!result.isEmpty())
+            return result;
+    }
+
+    // Fall back to ro.product.cpu.abi, ro.product.cpu.abi2 ...
     for (int i = 1; i < 6; ++i) {
         QStringList arguments = AndroidDeviceInfo::adbSelector(device);
         arguments << QLatin1String("shell") << QLatin1String("getprop");
@@ -854,15 +888,15 @@ SdkPlatform AndroidConfig::highestAndroidSdk() const
     return m_availableSdkPlatforms.first();
 }
 
-QString AndroidConfig::bestNdkPlatformMatch(const QString &targetAPI) const
+QString AndroidConfig::bestNdkPlatformMatch(int target) const
 {
+    target = std::max(9, target);
     updateNdkInformation();
-    int target = targetAPI.mid(targetAPI.lastIndexOf(QLatin1Char('-')) + 1).toInt();
     foreach (int apiLevel, m_availableNdkPlatforms) {
         if (apiLevel <= target)
             return QString::fromLatin1("android-%1").arg(apiLevel);
     }
-    return QLatin1String("android-8");
+    return QLatin1String("android-9");
 }
 
 FileName AndroidConfig::sdkLocation() const
@@ -949,6 +983,16 @@ void AndroidConfig::setAutomaticKitCreation(bool b)
     m_automaticKitCreation = b;
 }
 
+bool AndroidConfig::useGrandle() const
+{
+    return m_useGradle;
+}
+
+void AndroidConfig::setUseGradle(bool b)
+{
+    m_useGradle = b;
+}
+
 ///////////////////////////////////
 // AndroidConfigurations
 ///////////////////////////////////
@@ -963,7 +1007,9 @@ void AndroidConfigurations::setConfig(const AndroidConfig &devConfigs)
     emit m_instance->updated();
 }
 
-AndroidDeviceInfo AndroidConfigurations::showDeviceDialog(ProjectExplorer::Project *project, int apiLevel, const QString &abi)
+AndroidDeviceInfo AndroidConfigurations::showDeviceDialog(ProjectExplorer::Project *project,
+                                                          int apiLevel, const QString &abi,
+                                                          Options options)
 {
     QString serialNumber = defaultDevice(project, abi);
     if (!serialNumber.isEmpty()) {
@@ -979,7 +1025,7 @@ AndroidDeviceInfo AndroidConfigurations::showDeviceDialog(ProjectExplorer::Proje
                 return info;
     }
 
-    AndroidDeviceDialog dialog(apiLevel, abi, Core::ICore::mainWindow());
+    AndroidDeviceDialog dialog(apiLevel, abi, options, Core::ICore::mainWindow());
     if (dialog.exec() == QDialog::Accepted) {
         AndroidDeviceInfo info = dialog.device();
         if (dialog.saveDeviceSelection()) {
@@ -1163,7 +1209,7 @@ void AndroidConfigurations::updateAutomaticKitList()
     foreach (Kit *kit, newKits) {
         AndroidToolChain *tc = static_cast<AndroidToolChain *>(ToolChainKitInformation::toolChain(kit));
         AndroidQtVersion *qt = static_cast<AndroidQtVersion *>(QtSupport::QtKitInformation::qtVersion(kit));
-        kit->setDisplayName(tr("Android for %1 (GCC %2, Qt %3)")
+        kit->setUnexpandedDisplayName(tr("Android for %1 (GCC %2, Qt %3)")
                             .arg(qt->targetArch())
                             .arg(tc->ndkToolChainVersion())
                             .arg(qt->qtVersionString()));
@@ -1171,6 +1217,11 @@ void AndroidConfigurations::updateAutomaticKitList()
     }
 }
 
+bool AndroidConfigurations::force32bitEmulator()
+{
+    return m_instance->m_force32bit;
+}
+
 /**
  * Workaround for '????????????' serial numbers
  * @return ("-d") for buggy devices, ("-s", <serial no>) for normal
@@ -1208,25 +1259,27 @@ AndroidConfigurations::AndroidConfigurations(QObject *parent)
     connect(ProjectExplorer::SessionManager::instance(), SIGNAL(projectRemoved(ProjectExplorer::Project*)),
             this, SLOT(clearDefaultDevices(ProjectExplorer::Project*)));
 
+    m_force32bit = is32BitUserSpace();
+
     m_instance = this;
 }
 
-Utils::FileName javaHomeForJavac(const QString &location)
+static FileName javaHomeForJavac(const FileName &location)
 {
-    QFileInfo fileInfo(location);
+    QFileInfo fileInfo = location.toFileInfo();
     int tries = 5;
     while (tries > 0) {
         QDir dir = fileInfo.dir();
         dir.cdUp();
-        if (QFileInfo(dir.filePath(QLatin1String("lib/tools.jar"))).exists())
-            return Utils::FileName::fromString(dir.path());
+        if (QFileInfo::exists(dir.filePath(QLatin1String("lib/tools.jar"))))
+            return FileName::fromString(dir.path());
         if (fileInfo.isSymLink())
             fileInfo.setFile(fileInfo.symLinkTarget());
         else
             break;
         --tries;
     }
-    return Utils::FileName();
+    return FileName();
 }
 
 void AndroidConfigurations::load()
@@ -1238,10 +1291,10 @@ void AndroidConfigurations::load()
 
     if (m_config.antLocation().isEmpty()) {
         Environment env = Environment::systemEnvironment();
-        QString location = env.searchInPath(QLatin1String("ant"));
-        QFileInfo fi(location);
+        FileName location = env.searchInPath(QLatin1String("ant"));
+        QFileInfo fi = location.toFileInfo();
         if (fi.exists() && fi.isExecutable() && !fi.isDir()) {
-            m_config.setAntLocation(FileName::fromString(location));
+            m_config.setAntLocation(location);
             saveSettings = true;
         }
     }
@@ -1249,15 +1302,15 @@ void AndroidConfigurations::load()
     if (m_config.openJDKLocation().isEmpty()) {
         if (HostOsInfo::isLinuxHost()) {
             Environment env = Environment::systemEnvironment();
-            QString location = env.searchInPath(QLatin1String("javac"));
-            QFileInfo fi(location);
+            FileName location = env.searchInPath(QLatin1String("javac"));
+            QFileInfo fi = location.toFileInfo();
             if (fi.exists() && fi.isExecutable() && !fi.isDir()) {
                 m_config.setOpenJDKLocation(javaHomeForJavac(location));
                 saveSettings = true;
             }
         } else if (HostOsInfo::isMacHost()) {
             QString javaHome = QLatin1String("/System/Library/Frameworks/JavaVM.framework/Versions/CurrentJDK/Home");
-            if (QFileInfo(javaHome).exists())
+            if (QFileInfo::exists(javaHome))
                 m_config.setOpenJDKLocation(Utils::FileName::fromString(javaHome));
         } else if (HostOsInfo::isWindowsHost()) {
             QSettings settings(QLatin1String("HKEY_LOCAL_MACHINE\\SOFTWARE\\Javasoft\\Java Development Kit"), QSettings::NativeFormat);
@@ -1266,7 +1319,7 @@ void AndroidConfigurations::load()
             int major = -1;
             int minor = -1;
             foreach (const QString &version, allVersions) {
-                QStringList parts = version.split(QLatin1String("."));
+                QStringList parts = version.split(QLatin1Char('.'));
                 if (parts.size() != 2) // not interested in 1.7.0_u21
                     continue;
                 bool okMajor, okMinor;
@@ -1280,7 +1333,7 @@ void AndroidConfigurations::load()
                     settings.beginGroup(version);
                     QString tmpJavaHome = settings.value(QLatin1String("JavaHome")).toString();
                     settings.endGroup();
-                    if (!QFileInfo(tmpJavaHome).exists())
+                    if (!QFileInfo::exists(tmpJavaHome))
                         continue;
 
                     major = tmpMajor;
@@ -1304,7 +1357,7 @@ void AndroidConfigurations::load()
 void AndroidConfigurations::updateAndroidDevice()
 {
     DeviceManager * const devMgr = DeviceManager::instance();
-    if (m_instance->m_config.adbToolPath().toFileInfo().exists())
+    if (m_instance->m_config.adbToolPath().exists())
         devMgr->addDevice(IDevice::Ptr(new Internal::AndroidDevice));
     else if (devMgr->find(Constants::ANDROID_DEVICE_ID))
         devMgr->removeDevice(Core::Id(Constants::ANDROID_DEVICE_ID));
@@ -1312,5 +1365,4 @@ void AndroidConfigurations::updateAndroidDevice()
 
 AndroidConfigurations *AndroidConfigurations::m_instance = 0;
 
-} // namespace Internal
 } // namespace Android
