@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -184,7 +184,7 @@ public:
             QString lineText;
             if (node->incompatibleReason().isEmpty()) {
                 lineText = AndroidManager::androidNameForApiLevel(device.sdk) + QLatin1String("  ");
-                lineText += AndroidDeviceDialog::tr("ABI:") + device.cpuAbi.join(QLatin1String(" "));
+                lineText += AndroidDeviceDialog::tr("ABI:") + device.cpuAbi.join(QLatin1Char(' '));
             } else {
                 lineText = node->incompatibleReason();
             }
@@ -213,7 +213,7 @@ public:
 class AndroidDeviceModel : public QAbstractItemModel
 {
 public:
-    AndroidDeviceModel(int apiLevel, const QString &abi);
+    AndroidDeviceModel(int apiLevel, const QString &abi, AndroidConfigurations::Options options);
     QModelIndex index(int row, int column,
                       const QModelIndex &parent = QModelIndex()) const;
     QModelIndex parent(const QModelIndex &child) const;
@@ -229,6 +229,7 @@ public:
 private:
     int m_apiLevel;
     QString m_abi;
+    AndroidConfigurations::Options m_options;
     AndroidDeviceModelNode *m_root;
 };
 
@@ -237,8 +238,8 @@ private:
 /////////////////
 // AndroidDeviceModel
 /////////////////
-AndroidDeviceModel::AndroidDeviceModel(int apiLevel, const QString &abi)
-    : m_apiLevel(apiLevel), m_abi(abi), m_root(0)
+AndroidDeviceModel::AndroidDeviceModel(int apiLevel, const QString &abi, AndroidConfigurations::Options options)
+    : m_apiLevel(apiLevel), m_abi(abi), m_options(options), m_root(0)
 {
 }
 
@@ -343,10 +344,12 @@ void AndroidDeviceModel::setDevices(const QVector<AndroidDeviceInfo> &devices)
                     .arg(device.serialNumber);
         } else if (!device.cpuAbi.contains(m_abi)) {
             error = AndroidDeviceDialog::tr("ABI is incompatible, device supports ABIs: %1.")
-                    .arg(device.cpuAbi.join(QLatin1String(" ")));
+                    .arg(device.cpuAbi.join(QLatin1Char(' ')));
         } else if (device.sdk < m_apiLevel) {
             error = AndroidDeviceDialog::tr("API Level of device is: %1.")
                     .arg(device.sdk);
+        } else if (device.sdk > 20 && (m_options & AndroidConfigurations::FilterAndroid5)) {
+            error = AndroidDeviceDialog::tr("Android 5 devices are incompatible with deploying Qt to a temporary directory.");
         } else {
             new AndroidDeviceModelNode(compatibleDevices, device);
             continue;
@@ -386,9 +389,9 @@ static inline QString msgAdbListDevices()
     return AndroidDeviceDialog::tr("<p>The adb tool in the Android SDK lists all connected devices if run via &quot;adb devices&quot;.</p>");
 }
 
-AndroidDeviceDialog::AndroidDeviceDialog(int apiLevel, const QString &abi, QWidget *parent) :
+AndroidDeviceDialog::AndroidDeviceDialog(int apiLevel, const QString &abi, AndroidConfigurations::Options options, QWidget *parent) :
     QDialog(parent),
-    m_model(new AndroidDeviceModel(apiLevel, abi)),
+    m_model(new AndroidDeviceModel(apiLevel, abi, options)),
     m_ui(new Ui::AndroidDeviceDialog),
     m_apiLevel(apiLevel),
     m_abi(abi)
@@ -415,6 +418,8 @@ AndroidDeviceDialog::AndroidDeviceDialog(int apiLevel, const QString &abi, QWidg
 
     connect(m_ui->createAVDButton, SIGNAL(clicked()),
             this, SLOT(createAvd()));
+    connect(m_ui->deviceView, SIGNAL(doubleClicked(QModelIndex)),
+            this, SLOT(accept()));
 
     connect(&m_futureWatcher, SIGNAL(finished()),
             this, SLOT(avdAdded()));
