@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,20 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -32,21 +33,23 @@
 #include "androidconstants.h"
 #include "androidmanifestdocument.h"
 #include "androidmanager.h"
+#include "androidqtsupport.h"
 
+#include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/infobar.h>
 #include <coreplugin/editormanager/ieditor.h>
-#include <texteditor/plaintexteditor.h>
+
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectwindow.h>
-#include <projectexplorer/iprojectproperties.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/kitinformation.h>
+
 #include <texteditor/texteditoractionhandler.h>
-#include <texteditor/texteditorsettings.h>
-#include <qmakeprojectmanager/qmakeproject.h>
+#include <texteditor/texteditor.h>
+#include <utils/algorithm.h>
 
 #include <QLineEdit>
 #include <QFileInfo>
@@ -68,12 +71,14 @@
 #include <QTimer>
 #include <QCheckBox>
 
+#include <limits>
+
 using namespace ProjectExplorer;
 using namespace Android;
 using namespace Android::Internal;
 
 namespace {
-const QLatin1String packageNameRegExp("^([a-z_]{1}[a-z0-9_]+(\\.[a-zA-Z_]{1}[a-zA-Z0-9_]*)*)$");
+const QLatin1String packageNameRegExp("^([a-z]{1}[a-z0-9_]+(\\.[a-zA-Z]{1}[a-zA-Z0-9_]*)*)$");
 const char infoBarId[] = "Android.AndroidManifestEditor.InfoBar";
 const char androidManifestEditorGeneralPaneContextId[] = "AndroidManifestEditorWidget.GeneralWidget";
 
@@ -90,7 +95,7 @@ Project *androidProject(const QString &file)
             continue;
         Kit *kit = project->activeTarget()->kit();
         if (DeviceTypeKitInformation::deviceTypeId(kit) == Android::Constants::ANDROID_DEVICE_TYPE
-                && fileName.isChildOf(Utils::FileName::fromString(project->projectDirectory())))
+                && fileName.isChildOf(project->projectDirectory()))
             return project;
     }
     return 0;
@@ -106,7 +111,7 @@ AndroidManifestEditorWidget::AndroidManifestEditorWidget()
       m_appNameInStringsXml(false)
 {
     m_textEditorWidget = new AndroidManifestTextEditorWidget(this);
-    TextEditor::TextEditorSettings::initializeEditor(m_textEditorWidget);
+    m_textEditorWidget->setupAsPlainEditor();
 
     initializePage();
 
@@ -165,7 +170,7 @@ void AndroidManifestEditorWidget::initializePage()
         m_packageNameWarning->setVisible(false);
 
         m_packageNameWarningIcon = new QLabel;
-        m_packageNameWarningIcon->setPixmap(QPixmap(QString::fromUtf8(":/projectexplorer/images/compile_warning.png")));
+        m_packageNameWarningIcon->setPixmap(QPixmap(QLatin1String(Core::Constants::ICON_WARNING)));
         m_packageNameWarningIcon->setVisible(false);
         m_packageNameWarningIcon->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
 
@@ -178,7 +183,7 @@ void AndroidManifestEditorWidget::initializePage()
 
 
         m_versionCode = new QSpinBox(packageGroupBox);
-        m_versionCode->setMaximum(99);
+        m_versionCode->setMaximum(std::numeric_limits<int>::max());
         m_versionCode->setValue(1);
         m_versionCode->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
         formLayout->addRow(tr("Version code:"), m_versionCode);
@@ -280,20 +285,23 @@ void AndroidManifestEditorWidget::initializePage()
         QGridLayout *layout = new QGridLayout(permissionsGroupBox);
 
         m_defaultPermissonsCheckBox = new QCheckBox(this);
-        m_defaultPermissonsCheckBox->setText(tr("Include default permissions and features for Qt modules."));
-        m_defaultPermissonsCheckBox->setTristate(true);
+        m_defaultPermissonsCheckBox->setText(tr("Include default permissions for Qt modules."));
         layout->addWidget(m_defaultPermissonsCheckBox, 0, 0);
 
+        m_defaultFeaturesCheckBox = new QCheckBox(this);
+        m_defaultFeaturesCheckBox->setText(tr("Include default features for Qt modules."));
+        layout->addWidget(m_defaultFeaturesCheckBox, 1, 0);
+
         m_permissionsModel = new PermissionsModel(this);
 
         m_permissionsListView = new QListView(permissionsGroupBox);
         m_permissionsListView->setModel(m_permissionsModel);
         m_permissionsListView->setMinimumSize(QSize(0, 200));
-        layout->addWidget(m_permissionsListView, 1, 0, 3, 1);
+        layout->addWidget(m_permissionsListView, 2, 0, 3, 1);
 
         m_removePermissionButton = new QPushButton(permissionsGroupBox);
         m_removePermissionButton->setText(tr("Remove"));
-        layout->addWidget(m_removePermissionButton, 1, 1);
+        layout->addWidget(m_removePermissionButton, 2, 1);
 
         m_permissionsComboBox = new QComboBox(permissionsGroupBox);
         m_permissionsComboBox->insertItems(0, QStringList()
@@ -429,16 +437,18 @@ void AndroidManifestEditorWidget::initializePage()
          << QLatin1String("android.permission.WRITE_USER_DICTIONARY")
         );
         m_permissionsComboBox->setEditable(true);
-        layout->addWidget(m_permissionsComboBox, 5, 0);
+        layout->addWidget(m_permissionsComboBox, 6, 0);
 
         m_addPermissionButton = new QPushButton(permissionsGroupBox);
         m_addPermissionButton->setText(tr("Add"));
-        layout->addWidget(m_addPermissionButton, 5, 1);
+        layout->addWidget(m_addPermissionButton, 6, 1);
 
         permissionsGroupBox->setLayout(layout);
 
         connect(m_defaultPermissonsCheckBox, SIGNAL(stateChanged(int)),
-                this, SLOT(defaultPermissionCheckBoxClicked()));
+                this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
+        connect(m_defaultFeaturesCheckBox, SIGNAL(stateChanged(int)),
+                this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
 
         connect(m_addPermissionButton, SIGNAL(clicked()),
                 this, SLOT(addPermission()));
@@ -466,13 +476,13 @@ bool AndroidManifestEditorWidget::eventFilter(QObject *obj, QEvent *event)
 
 void AndroidManifestEditorWidget::updateTargetComboBox()
 {
-    const QString docPath(m_textEditorWidget->baseTextDocument()->filePath());
+    const QString docPath(m_textEditorWidget->textDocument()->filePath());
     ProjectExplorer::Project *project = androidProject(docPath);
     QStringList items;
     if (project) {
         ProjectExplorer::Kit *kit = project->activeTarget()->kit();
         if (ProjectExplorer::DeviceTypeKitInformation::deviceTypeId(kit) == Constants::ANDROID_DEVICE_TYPE)
-            items = AndroidManager::availableTargetApplications(project->activeTarget());
+            items = AndroidManager::androidQtSupport(project->activeTarget())->projectTargetApplications(project->activeTarget());
     }
 
     // QComboBox randomly resets what the user has entered
@@ -567,7 +577,7 @@ void AndroidManifestEditorWidget::preSave()
         syncToEditor();
 
     if (m_setAppName && m_appNameInStringsXml) {
-        QString baseDir = QFileInfo(m_textEditorWidget->baseTextDocument()->filePath()).absolutePath();
+        QString baseDir = QFileInfo(m_textEditorWidget->textDocument()->filePath()).absolutePath();
         QString fileName = baseDir + QLatin1String("/res/values/strings.xml");
         QFile f(fileName);
         if (f.open(QIODevice::ReadOnly)) {
@@ -591,7 +601,7 @@ void AndroidManifestEditorWidget::preSave()
         m_setAppName = false;
     }
 
-    QString baseDir = QFileInfo(m_textEditorWidget->baseTextDocument()->filePath()).absolutePath();
+    QString baseDir = QFileInfo(m_textEditorWidget->textDocument()->filePath()).absolutePath();
     if (!m_lIconPath.isEmpty()) {
         copyIcon(LowDPI, baseDir, m_lIconPath);
         m_lIconPath.clear();
@@ -614,7 +624,7 @@ Core::IEditor *AndroidManifestEditorWidget::editor() const
     return m_editor;
 }
 
-TextEditor::PlainTextEditorWidget *AndroidManifestEditorWidget::textEditorWidget() const
+TextEditor::TextEditorWidget *AndroidManifestEditorWidget::textEditorWidget() const
 {
     return m_textEditorWidget;
 }
@@ -685,9 +695,8 @@ void AndroidManifestEditorWidget::updateInfoBar()
 
 void AndroidManifestEditorWidget::updateSdkVersions()
 {
-    const QString docPath(m_textEditorWidget->baseTextDocument()->filePath());
-    Project *project = androidProject(docPath);
-    QPair<int, int> apiLevels = AndroidManager::apiLevelRange(project ? project->activeTarget() : 0);
+    const QString docPath(m_textEditorWidget->textDocument()->filePath());
+    QPair<int, int> apiLevels = AndroidManager::apiLevelRange();
     for (int i = apiLevels.first; i < apiLevels.second + 1; ++i)
         m_androidMinSdkVersion->addItem(tr("API %1: %2")
                                         .arg(i)
@@ -703,14 +712,16 @@ void AndroidManifestEditorWidget::updateSdkVersions()
 
 void AndroidManifestEditorWidget::updateInfoBar(const QString &errorMessage, int line, int column)
 {
-    Core::InfoBar *infoBar = m_textEditorWidget->baseTextDocument()->infoBar();
+    Core::InfoBar *infoBar = m_textEditorWidget->textDocument()->infoBar();
     QString text;
     if (line < 0)
-        text = tr("Could not parse file: '%1'.").arg(errorMessage);
+        text = tr("Could not parse file: \"%1\".").arg(errorMessage);
     else
-        text = tr("%2: Could not parse file: '%1'.").arg(errorMessage).arg(line);
+        text = tr("%2: Could not parse file: \"%1\".").arg(errorMessage).arg(line);
     Core::InfoBarEntry infoBarEntry(infoBarId, text);
-    infoBarEntry.setCustomButtonInfo(tr("Goto error"), this, SLOT(gotoError()));
+    infoBarEntry.setCustomButtonInfo(tr("Goto error"), [this]() {
+        m_textEditorWidget->gotoLine(m_errorLine, m_errorColumn);
+    });
     infoBar->removeInfo(infoBarId);
     infoBar->addInfo(infoBarEntry);
 
@@ -721,16 +732,11 @@ void AndroidManifestEditorWidget::updateInfoBar(const QString &errorMessage, int
 
 void AndroidManifestEditorWidget::hideInfoBar()
 {
-    Core::InfoBar *infoBar = m_textEditorWidget->baseTextDocument()->infoBar();
+    Core::InfoBar *infoBar = m_textEditorWidget->textDocument()->infoBar();
         infoBar->removeInfo(infoBarId);
     m_timerParseCheck.stop();
 }
 
-void AndroidManifestEditorWidget::gotoError()
-{
-    m_textEditorWidget->gotoLine(m_errorLine, m_errorColumn);
-}
-
 void setApiLevel(QComboBox *box, const QDomElement &element, const QString &attribute)
 {
     if (!element.isNull() && element.hasAttribute(attribute)) {
@@ -760,7 +766,7 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     setApiLevel(m_androidMinSdkVersion, usesSdkElement, QLatin1String("android:minSdkVersion"));
     setApiLevel(m_androidTargetSdkVersion, usesSdkElement, QLatin1String("android:targetSdkVersion"));
 
-    QString baseDir = QFileInfo(m_textEditorWidget->baseTextDocument()->filePath()).absolutePath();
+    QString baseDir = QFileInfo(m_textEditorWidget->textDocument()->filePath()).absolutePath();
     QString fileName = baseDir + QLatin1String("/res/values/strings.xml");
 
     QDomElement applicationElement = manifest.firstChildElement(QLatin1String("application"));
@@ -801,9 +807,12 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
     m_hIconPath.clear();
 
     disconnect(m_defaultPermissonsCheckBox, SIGNAL(stateChanged(int)),
-            this, SLOT(defaultPermissionCheckBoxClicked()));
+            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
+    disconnect(m_defaultFeaturesCheckBox, SIGNAL(stateChanged(int)),
+            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
 
     m_defaultPermissonsCheckBox->setChecked(false);
+    m_defaultFeaturesCheckBox->setChecked(false);
     QDomNodeList manifestChilds = manifest.childNodes();
     bool foundPermissionComment = false;
     bool foundFeatureComment = false;
@@ -818,10 +827,13 @@ void AndroidManifestEditorWidget::syncToWidgets(const QDomDocument &doc)
         }
     }
 
-    m_defaultPermissonsCheckBox->setCheckState(Qt::CheckState(foundFeatureComment + foundPermissionComment));
+    m_defaultPermissonsCheckBox->setChecked(foundPermissionComment);
+    m_defaultFeaturesCheckBox->setChecked(foundFeatureComment);
 
     connect(m_defaultPermissonsCheckBox, SIGNAL(stateChanged(int)),
-            this, SLOT(defaultPermissionCheckBoxClicked()));
+            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
+    connect(m_defaultFeaturesCheckBox, SIGNAL(stateChanged(int)),
+            this, SLOT(defaultPermissionOrFeatureCheckBoxClicked()));
 
     QStringList permissions;
     QDomElement permissionElem = manifest.firstChildElement(QLatin1String("uses-permission"));
@@ -844,11 +856,7 @@ int extractVersion(const QString &string)
     int index = string.indexOf(QLatin1Char(':'));
     if (index == -1)
         return 0;
-#if QT_VERSION < 0x050100
-    return string.mid(4, index - 4).toInt();
-#else
     return string.midRef(4, index - 4).toInt();
-#endif
 }
 
 void AndroidManifestEditorWidget::syncToEditor()
@@ -963,7 +971,7 @@ void AndroidManifestEditorWidget::parseManifest(QXmlStreamReader &reader, QXmlSt
             if (!foundPermissionComment && m_defaultPermissonsCheckBox->checkState() == Qt::Checked)
                 writer.writeComment(QLatin1String(" %%INSERT_PERMISSIONS "));
 
-            if (!foundFeatureComment && m_defaultPermissonsCheckBox->checkState() == Qt::Checked)
+            if (!foundFeatureComment && m_defaultFeaturesCheckBox->checkState() == Qt::Checked)
                 writer.writeComment(QLatin1String(" %%INSERT_FEATURES "));
 
             if (!permissions.isEmpty()) {
@@ -1001,6 +1009,7 @@ void AndroidManifestEditorWidget::parseApplication(QXmlStreamReader &reader, QXm
     if (!m_appNameInStringsXml) {
         keys << QLatin1String("android:label");
         values << m_appNameLineEdit->text();
+        m_setAppName = false;
     }
     bool ensureIconAttribute =  !m_lIconPath.isEmpty()
             || !m_mIconPath.isEmpty()
@@ -1178,16 +1187,18 @@ QString AndroidManifestEditorWidget::parseUsesPermission(QXmlStreamReader &reade
 QString AndroidManifestEditorWidget::parseComment(QXmlStreamReader &reader, QXmlStreamWriter &writer)
 {
     QString commentText = reader.text().toString().trimmed();
-    if (commentText == QLatin1String("%%INSERT_PERMISSIONS")
-            || commentText == QLatin1String("%%INSERT_FEATURES")) {
+    if (commentText == QLatin1String("%%INSERT_PERMISSIONS")) {
         if (m_defaultPermissonsCheckBox->checkState() == Qt::Unchecked)
             return commentText;
-
-        writer.writeCurrentToken(reader);
-        return commentText;
     }
+
+    if (commentText == QLatin1String("%%INSERT_FEATURES")) {
+        if (m_defaultFeaturesCheckBox->checkState() == Qt::Unchecked)
+            return commentText;
+    }
+
     writer.writeCurrentToken(reader);
-    return QString();
+    return commentText;
 }
 
 void AndroidManifestEditorWidget::parseUnknownElement(QXmlStreamReader &reader, QXmlStreamWriter &writer)
@@ -1248,7 +1259,7 @@ QIcon AndroidManifestEditorWidget::icon(const QString &baseDir, IconDPI dpi)
 
 void AndroidManifestEditorWidget::copyIcon(IconDPI dpi, const QString &baseDir, const QString &filePath)
 {
-    if (!QFileInfo(filePath).exists())
+    if (!QFileInfo::exists(filePath))
         return;
 
     const QString targetPath = iconPath(baseDir, dpi);
@@ -1288,10 +1299,8 @@ void AndroidManifestEditorWidget::setHDPIIcon()
     setDirty(true);
 }
 
-void AndroidManifestEditorWidget::defaultPermissionCheckBoxClicked()
+void AndroidManifestEditorWidget::defaultPermissionOrFeatureCheckBoxClicked()
 {
-    if (m_defaultPermissonsCheckBox->checkState() == Qt::PartiallyChecked)
-        m_defaultPermissonsCheckBox->setChecked(true);
     setDirty(true);
 }
 
@@ -1347,7 +1356,7 @@ void PermissionsModel::setPermissions(const QStringList &permissions)
 {
     beginResetModel();
     m_permissions = permissions;
-    qSort(m_permissions);
+    Utils::sort(m_permissions);
     endResetModel();
 }
 
@@ -1417,9 +1426,10 @@ int PermissionsModel::rowCount(const QModelIndex &parent) const
 
 
 AndroidManifestTextEditorWidget::AndroidManifestTextEditorWidget(AndroidManifestEditorWidget *parent)
-    : TextEditor::PlainTextEditorWidget(new AndroidManifestDocument(parent), parent),
-      m_parent(parent)
+    : TextEditor::TextEditorWidget(parent)
 {
-    baseTextDocument()->setMimeType(QLatin1String(Constants::ANDROID_MANIFEST_MIME_TYPE));
+    setTextDocument(TextEditor::TextDocumentPtr(new AndroidManifestDocument(parent)));
+    setupAsPlainEditor();
+    textDocument()->setMimeType(QLatin1String(Constants::ANDROID_MANIFEST_MIME_TYPE));
 }
 
