@@ -1,8 +1,8 @@
 /**************************************************************************
 **
-** Copyright (c) 2014 AudioCodes Ltd.
+** Copyright (C) 2015 AudioCodes Ltd.
 ** Author: Orgad Shaneh <orgad.shaneh@audiocodes.com>
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -10,20 +10,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -54,13 +55,13 @@ class QTextCodec;
 QT_END_NAMESPACE
 
 namespace Core {
+class CommandLocator;
 class IVersionControl;
 class IEditor;
 } // namespace Core
 
 namespace Utils { class ParameterAction; }
 namespace VcsBase { class VcsBaseSubmitEditor; }
-namespace Locator { class CommandLocator; }
 namespace ProjectExplorer { class Project; }
 
 namespace ClearCase {
@@ -89,7 +90,8 @@ public:
         CheckedOut = 0x02,
         Hijacked   = 0x04,
         NotManaged = 0x08,
-        Missing    = 0x10
+        Missing    = 0x10,
+        Derived    = 0x20
     } status;
 
     QFile::Permissions permissions;
@@ -117,11 +119,7 @@ class ClearCasePlugin : public VcsBase::VcsBasePlugin
     Q_OBJECT
     Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QtCreatorPlugin" FILE "ClearCase.json")
 
-    static const unsigned SilentRun =
-        SuppressStdErrInLogWindow |
-        SuppressFailMessageInLogWindow |
-        SuppressCommandLogging |
-        FullySynchronously;
+    enum { SilentRun = NoOutput | FullySynchronously };
 
 public:
     ClearCasePlugin();
@@ -155,9 +153,11 @@ public:
     QString ccGetPredecessor(const QString &version) const;
     QStringList ccGetActiveVobs() const;
     ViewData ccGetView(const QString &workingDir) const;
+    QString ccGetComment(const QString &workingDir, const QString &fileName) const;
     bool ccFileOp(const QString &workingDir, const QString &title, const QStringList &args,
                   const QString &fileName, const QString &file2 = QString());
     FileStatus vcsStatus(const QString &file) const;
+    void checkAndReIndexUnknownFile(const QString &file);
     QString currentView() const { return m_viewData.name; }
     QString viewRoot() const { return m_viewData.root; }
     void refreshActivities();
@@ -167,6 +167,10 @@ public:
 
     bool ccCheckUcm(const QString &viewname, const QString &workingDir) const;
     bool managesFile(const QString &workingDirectory, const QString &fileName) const;
+#ifdef WITH_TESTS
+    inline void setFakeCleartool(const bool b = true) { m_fakeClearTool = b; }
+    inline bool isFakeCleartool() const { return m_fakeClearTool; }
+#endif
 
 public slots:
     void vcsAnnotate(const QString &workingDir, const QString &file,
@@ -199,9 +203,21 @@ private slots:
     void closing();
     void updateStatusActions();
 #ifdef WITH_TESTS
+    void initTestCase();
+    void cleanupTestCase();
     void testDiffFileResolving_data();
     void testDiffFileResolving();
     void testLogResolving();
+    void testFileStatusParsing_data();
+    void testFileStatusParsing();
+    void testFileNotManaged();
+    void testFileCheckedOutDynamicView();
+    void testFileCheckedInDynamicView();
+    void testFileNotManagedDynamicView();
+    void testStatusActions_data();
+    void testStatusActions();
+    void testVcsStatusDynamicReadonlyNotManaged();
+    void testVcsStatusDynamicNotManaged();
 #endif
 
 protected:
@@ -244,6 +260,10 @@ private:
                        int timeOut, QTextCodec *outputCodec = 0);
     static QString getDriveLetterOfPath(const QString &directory);
 
+    FileStatus::Status getFileStatus(const QString &fileName) const;
+    void updateStatusForFile(const QString &absFile);
+    void updateEditDerivedObjectWarning(const QString &fileName, const FileStatus::Status status);
+
     ClearCaseSettings m_settings;
 
     QString m_checkInMessageFileName;
@@ -255,7 +275,7 @@ private:
     QString m_activity;
     QString m_diffPrefix;
 
-    Locator::CommandLocator *m_commandLocator;
+    Core::CommandLocator *m_commandLocator;
     Utils::ParameterAction *m_checkOutAction;
     Utils::ParameterAction *m_checkInCurrentAction;
     Utils::ParameterAction *m_undoCheckOutAction;
@@ -282,6 +302,10 @@ private:
     QSharedPointer<StatusMap> m_statusMap;
 
     static ClearCasePlugin *m_clearcasePluginInstance;
+#ifdef WITH_TESTS
+    bool m_fakeClearTool;
+    QString m_tempFile;
+#endif
 };
 
 } // namespace Internal
