@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -38,6 +38,7 @@
 #include <coreplugin/dialogs/readonlyfilesdialog.h>
 #include <coreplugin/dialogs/saveitemsdialog.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/editormanager/editormanager_p.h>
 #include <coreplugin/editormanager/editorview.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/editormanager/ieditorfactory.h>
@@ -199,7 +200,7 @@ DocumentManagerPrivate::DocumentManagerPrivate() :
     m_linkWatcher(0),
     m_blockActivated(false),
     m_lastVisitedDirectory(QDir::currentPath()),
-    m_useProjectsDirectory(Utils::HostOsInfo::isMacHost()), // Creator is in bizarre places when launched via finder.
+    m_useProjectsDirectory(true),
     m_blockedIDocument(0)
 {
 }
@@ -228,7 +229,7 @@ DocumentManager::~DocumentManager()
     delete d;
 }
 
-QObject *DocumentManager::instance()
+DocumentManager *DocumentManager::instance()
 {
     return m_instance;
 }
@@ -609,7 +610,7 @@ static bool saveModifiedFilesHelper(const QList<IDocument *> &documents,
             roDialog.setShowFailWarning(true, DocumentManager::tr(
                                             "Could not save the files.",
                                             "error message"));
-            if (roDialog.exec() == Core::Internal::ReadOnlyFilesDialog::RO_Cancel) {
+            if (roDialog.exec() == ReadOnlyFilesDialog::RO_Cancel) {
                 if (cancelled)
                     (*cancelled) = true;
                 if (failedToSave)
@@ -618,7 +619,7 @@ static bool saveModifiedFilesHelper(const QList<IDocument *> &documents,
             }
         }
         foreach (IDocument *document, documentsToSave) {
-            if (!EditorManager::saveDocument(document)) {
+            if (!EditorManagerPrivate::saveDocument(document)) {
                 if (cancelled)
                     *cancelled = true;
                 notSaved.append(document);
@@ -679,8 +680,8 @@ QString DocumentManager::getSaveFileName(const QString &title, const QString &pa
                 // Mime database creates filter strings like this: Anything here (*.foo *.bar)
                 QRegExp regExp(QLatin1String(".*\\s+\\((.*)\\)$"));
                 const int index = regExp.lastIndexIn(*selectedFilter);
-                bool suffixOk = false;
                 if (index != -1) {
+                    bool suffixOk = false;
                     const QStringList &suffixes = regExp.cap(1).remove(QLatin1Char('*')).split(QLatin1Char(' '));
                     foreach (const QString &suffix, suffixes)
                         if (fileName.endsWith(suffix)) {
@@ -722,13 +723,16 @@ QString DocumentManager::getSaveAsFileName(const IDocument *document, const QStr
         return QLatin1String("");
     QString absoluteFilePath = document->filePath();
     const QFileInfo fi(absoluteFilePath);
-    QString fileName = fi.fileName();
-    QString path = fi.absolutePath();
+    QString path;
+    QString fileName;
     if (absoluteFilePath.isEmpty()) {
         fileName = document->suggestedFileName();
         const QString defaultPath = document->defaultPath();
         if (!defaultPath.isEmpty())
             path = defaultPath;
+    } else {
+        path = fi.absolutePath();
+        fileName = fi.fileName();
     }
 
     QString filterString;
@@ -904,17 +908,19 @@ void DocumentManager::checkForReload()
     if (!QApplication::activeWindow())
         return;
 
-    if (QApplication::activeModalWidget()) { // a modal dialog, recheck later
+    if (QApplication::activeModalWidget() || d->m_blockActivated) {
+        // We do not want to prompt for modified file if we currently have some modal dialog open.
+        // If d->m_blockActivated is true, then it means that the event processing of either the
+        // file modified dialog, or of loading large files, has delivered a file change event from
+        // a watcher *and* the timer triggered. We may never end up here in a nested way, so
+        // recheck later.
         QTimer::singleShot(200, this, SLOT(checkForReload()));
         return;
     }
 
-    if (d->m_blockActivated)
-        return;
-
     d->m_blockActivated = true;
 
-    IDocument::ReloadSetting defaultBehavior = EditorManager::reloadSetting();
+    IDocument::ReloadSetting defaultBehavior = EditorManagerPrivate::reloadSetting();
     Utils::ReloadPromptAnswer previousReloadAnswer = Utils::ReloadCurrent;
     Utils::FileDeletedPromptAnswer previousDeletedAnswer = Utils::FileDeletedSave;
 
@@ -1005,6 +1011,14 @@ void DocumentManager::checkForReload()
         // handle it!
         d->m_blockedIDocument = document;
 
+        // Update file info, also handling if e.g. link target has changed.
+        // We need to do that before the file is reloaded, because removing the watcher will
+        // loose any pending change events. Loosing change events *before* the file is reloaded
+        // doesn't matter, because in that case we then reload the new version of the file already
+        // anyhow.
+        removeFileInfo(document);
+        addFileInfo(document);
+
         bool success = true;
         QString errorString;
         // we've got some modification
@@ -1106,14 +1120,11 @@ void DocumentManager::checkForReload()
                 errorStrings << errorString;
         }
 
-        // update file info, also handling if e.g. link target has changed
-        removeFileInfo(document);
-        addFileInfo(document);
         d->m_blockedIDocument = 0;
     }
     if (!errorStrings.isEmpty())
         QMessageBox::critical(ICore::dialogParent(), tr("File Error"),
-                              errorStrings.join(QLatin1String("\n")));
+                              errorStrings.join(QLatin1Char('\n')));
 
     // handle deleted files
     EditorManager::closeDocuments(documentsToClose, false);
@@ -1422,7 +1433,7 @@ void DocumentManager::executeOpenWithMenuAction(QAction *action)
         QList<IEditor *> editorsOpenForFile
                 = DocumentModel::editorsForFilePath(entry.fileName);
         foreach (IEditor *openEditor, editorsOpenForFile) {
-            Internal::EditorView *view = EditorManager::viewForEditor(openEditor);
+            Internal::EditorView *view = EditorManagerPrivate::viewForEditor(openEditor);
             if (view && view->currentEditor() == openEditor) // visible
                 views.append(view);
         }
@@ -1432,15 +1443,15 @@ void DocumentManager::executeOpenWithMenuAction(QAction *action)
         if (views.isEmpty()) {
             EditorManager::openEditor(entry.fileName, entry.editorFactory->id());
         } else {
-            if (Internal::EditorView *currentView = EditorManager::currentEditorView()) {
+            if (Internal::EditorView *currentView = EditorManagerPrivate::currentEditorView()) {
                 if (views.removeOne(currentView))
                     views.prepend(currentView); // open editor in current view first
             }
             EditorManager::OpenEditorFlags flags;
             foreach (Internal::EditorView *view, views) {
                 IEditor *editor =
-                        EditorManager::openEditor(view, entry.fileName, entry.editorFactory->id(),
-                                                  flags);
+                        EditorManagerPrivate::openEditor(view, entry.fileName,
+                                                         entry.editorFactory->id(), flags);
                 // Do not change the current editor after opening the first one. That
                 // * prevents multiple updates of focus etc which are not necessary
                 // * lets us control which editor is made current by putting the current editor view
