@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -43,6 +43,7 @@
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/stylehelper.h>
+#include <utils/theme/theme.h>
 
 #include <QAction>
 #include <QEvent>
@@ -62,6 +63,7 @@ static const char kDetailsPinned[] = "DetailsPinned";
 
 using namespace Core;
 using namespace Core::Internal;
+using namespace Utils;
 
 /*!
     \mainclass
@@ -283,6 +285,7 @@ ProgressManagerPrivate::ProgressManagerPrivate()
 
 ProgressManagerPrivate::~ProgressManagerPrivate()
 {
+    stopFadeOfSummaryProgress();
     qDeleteAll(m_taskList);
     m_taskList.clear();
     ExtensionSystem::PluginManager::removeObject(m_statusBarWidgetContainer);
@@ -703,22 +706,26 @@ ToggleButton::ToggleButton(QWidget *parent)
     : QToolButton(parent)
 {
     setToolButtonStyle(Qt::ToolButtonIconOnly);
+    if (creatorTheme()->widgetStyle() == Theme::StyleFlat) {
+        QPalette p = palette();
+        p.setBrush(QPalette::Base, creatorTheme()->color(Theme::ToggleButtonBackgroundColor));
+        setPalette(p);
+    }
 }
 
 QSize ToggleButton::sizeHint() const
 {
-    return QSize(12, 12);
+    return QSize(13, 12); // Uneven width, because the arrow's width is also uneven.
 }
 
 void ToggleButton::paintEvent(QPaintEvent *event)
 {
     QToolButton::paintEvent(event);
     QPainter p(this);
-    QStyle *s = style();
     QStyleOption arrowOpt;
     arrowOpt.initFrom(this);
-    arrowOpt.rect = QRect(rect().center().x() - 3, rect().center().y() - 6, 9, 9);
-    s->drawPrimitive(QStyle::PE_IndicatorArrowUp, &arrowOpt, &p, this);
+    arrowOpt.rect.adjust(2, 0, -1, -2);
+    Utils::StyleHelper::drawArrow(QStyle::PE_IndicatorArrowUp, &p, &arrowOpt);
 }
 
 
