@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -54,8 +54,7 @@ class SimpleDiffEditorReloader : public DiffEditorReloader
 {
     Q_OBJECT
 public:
-    SimpleDiffEditorReloader(QObject *parent,
-                             const QString &leftFileName,
+    SimpleDiffEditorReloader(const QString &leftFileName,
                              const QString &rightFileName);
 
 protected:
@@ -66,11 +65,9 @@ private:
     QString m_rightFileName;
 };
 
-SimpleDiffEditorReloader::SimpleDiffEditorReloader(QObject *parent,
-                                                   const QString &leftFileName,
+SimpleDiffEditorReloader::SimpleDiffEditorReloader(const QString &leftFileName,
                                                    const QString &rightFileName)
-    : DiffEditorReloader(parent),
-      m_leftFileName(leftFileName),
+    : m_leftFileName(leftFileName),
       m_rightFileName(rightFileName)
 {
 }
@@ -197,16 +194,15 @@ void DiffEditorPlugin::diff()
 
     const QString documentId = QLatin1String("Diff ") + fileName1
             + QLatin1String(", ") + fileName2;
-    DiffEditorDocument *document = DiffEditorManager::find(documentId);
     QString title = tr("Diff \"%1\", \"%2\"").arg(fileName1).arg(fileName2);
-    document = DiffEditorManager::findOrCreate(documentId, title);
+    DiffEditorDocument * const document = DiffEditorManager::findOrCreate(documentId, title);
     if (!document)
         return;
 
     DiffEditorController *controller = document->controller();
     if (!controller->reloader()) {
         SimpleDiffEditorReloader *reloader =
-                new SimpleDiffEditorReloader(controller, fileName1, fileName2);
+                new SimpleDiffEditorReloader(fileName1, fileName2);
         controller->setReloader(reloader);
     }
 
@@ -475,7 +471,7 @@ void DiffEditor::Internal::DiffEditorPlugin::testMakePatch()
     QCOMPARE(result, patchText);
 
     bool ok;
-    QList<FileData> resultList = DiffUtils::readPatch(result, false, &ok);
+    QList<FileData> resultList = DiffUtils::readPatch(result, &ok);
 
     QVERIFY(ok);
     QCOMPARE(resultList.count(), 1);
@@ -821,6 +817,82 @@ void DiffEditor::Internal::DiffEditorPlugin::testReadPatch_data()
 
     QTest::newRow("Blank line followed by No newline") << patch
                                 << fileDataList5;
+
+    //////////////
+
+    // Based on 953cdb97
+    patch = _("diff --git a/src/plugins/texteditor/basetextdocument.h b/src/plugins/texteditor/textdocument.h\n"
+              "similarity index 100%\n"
+              "rename from src/plugins/texteditor/basetextdocument.h\n"
+              "rename to src/plugins/texteditor/textdocument.h\n"
+              "diff --git a/src/plugins/texteditor/basetextdocumentlayout.cpp b/src/plugins/texteditor/textdocumentlayout.cpp\n"
+              "similarity index 79%\n"
+              "rename from src/plugins/texteditor/basetextdocumentlayout.cpp\n"
+              "rename to src/plugins/texteditor/textdocumentlayout.cpp\n"
+              "index 0121933..01cc3a0 100644\n"
+              "--- a/src/plugins/texteditor/basetextdocumentlayout.cpp\n"
+              "+++ b/src/plugins/texteditor/textdocumentlayout.cpp\n"
+              "@@ -2,5 +2,5 @@ void func()\n"
+              " A\n"
+              " B\n"
+              "-C\n"
+              "+Z\n"
+              " D\n"
+              " \n"
+              );
+
+    fileData1 = FileData();
+    fileData1.leftFileInfo = DiffFileInfo(_("src/plugins/texteditor/basetextdocument.h"));
+    fileData1.rightFileInfo = DiffFileInfo(_("src/plugins/texteditor/textdocument.h"));
+    fileData1.fileOperation = FileData::RenameFile;
+    fileData2 = FileData();
+    fileData2.leftFileInfo = DiffFileInfo(_("src/plugins/texteditor/basetextdocumentlayout.cpp"), _("0121933"));
+    fileData2.rightFileInfo = DiffFileInfo(_("src/plugins/texteditor/textdocumentlayout.cpp"), _("01cc3a0"));
+    fileData2.fileOperation = FileData::RenameFile;
+    chunkData2.leftStartingLineNumber = 1;
+    chunkData2.rightStartingLineNumber = 1;
+    rows2.clear();
+    rows2 << RowData(_("A"));
+    rows2 << RowData(_("B"));
+    rows2 << RowData(_("C"), _("Z"));
+    rows2 << RowData(_("D"));
+    rows2 << RowData(_(""));
+    chunkData2.rows = rows2;
+    fileData2.chunks.clear();
+    fileData2.chunks << chunkData2;
+
+    QList<FileData> fileDataList6;
+    fileDataList6 << fileData1 << fileData2;
+
+    QTest::newRow("Multiple renames") << patch
+                                      << fileDataList6;
+
+    //////////////
+
+    // Dirty submodule
+    patch = _("diff --git a/src/shared/qbs b/src/shared/qbs\n"
+              "--- a/src/shared/qbs\n"
+              "+++ b/src/shared/qbs\n"
+              "@@ -1 +1 @@\n"
+              "-Subproject commit eda76354077a427d692fee05479910de31040d3f\n"
+              "+Subproject commit eda76354077a427d692fee05479910de31040d3f-dirty\n"
+              );
+    fileData1 = FileData();
+    fileData1.leftFileInfo = DiffFileInfo(_("src/shared/qbs"));
+    fileData1.rightFileInfo = DiffFileInfo(_("src/shared/qbs"));
+    chunkData1.leftStartingLineNumber = 0;
+    chunkData1.rightStartingLineNumber = 0;
+    rows1.clear();
+    rows1 << RowData(_("Subproject commit eda76354077a427d692fee05479910de31040d3f"),
+                     _("Subproject commit eda76354077a427d692fee05479910de31040d3f-dirty"));
+    chunkData1.rows = rows1;
+    fileData1.chunks.clear();
+    fileData1.chunks <<  chunkData1;
+
+    QList<FileData> fileDataList7;
+    fileDataList7 << fileData1;
+    QTest::newRow("Dirty submodule") << patch
+                                     << fileDataList7;
 }
 
 void DiffEditor::Internal::DiffEditorPlugin::testReadPatch()
@@ -829,7 +901,7 @@ void DiffEditor::Internal::DiffEditorPlugin::testReadPatch()
     QFETCH(QList<FileData>, fileDataList);
 
     bool ok;
-    QList<FileData> result = DiffUtils::readPatch(sourcePatch, false, &ok);
+    QList<FileData> result = DiffUtils::readPatch(sourcePatch, &ok);
 
     QVERIFY(ok);
     QCOMPARE(fileDataList.count(), result.count());
@@ -864,7 +936,4 @@ void DiffEditor::Internal::DiffEditorPlugin::testReadPatch()
 
 #endif // WITH_TESTS
 
-
-Q_EXPORT_PLUGIN(DiffEditor::Internal::DiffEditorPlugin)
-
 #include "diffeditorplugin.moc"
