@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -32,11 +32,13 @@
 #include "gitclient.h"
 
 #include <utils/qtcassert.h>
-#include <vcsbase/vcsbaseoutputwindow.h>
+#include <vcsbase/vcsoutputwindow.h>
 #include <vcsbase/vcsbaseplugin.h>
 
 #include <QFont>
 
+using namespace VcsBase;
+
 namespace Git {
 namespace Internal {
 
@@ -172,7 +174,7 @@ public:
             }
             return names;
         }
-        return QStringList(fullName().join(QString(QLatin1Char('/'))));
+        return QStringList(fullName().join(QLatin1Char('/')));
     }
 
     int rowOf(BranchNode *node)
@@ -311,7 +313,7 @@ bool BranchModel::setData(const QModelIndex &index, const QVariant &value, int r
                                                       << newFullName.last(),
                                         &output, &errorMessage)) {
         node->name = oldFullName.last();
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
         return false;
     }
 
@@ -355,7 +357,7 @@ bool BranchModel::refresh(const QString &workingDirectory, QString *errorMessage
     args << QLatin1String("--format=%(objectname)\t%(refname)\t%(upstream:short)\t%(*objectname)");
     QString output;
     if (!m_client->synchronousForEachRefCmd(workingDirectory, args, &output, errorMessage))
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(*errorMessage);
+        VcsOutputWindow::appendError(*errorMessage);
 
     m_workingDirectory = workingDirectory;
     const QStringList lines = output.split(QLatin1Char('\n'));
@@ -394,7 +396,7 @@ void BranchModel::renameBranch(const QString &oldName, const QString &newName)
     if (!m_client->synchronousBranchCmd(m_workingDirectory,
                                         QStringList() << QLatin1String("-m") << oldName << newName,
                                         &output, &errorMessage))
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
     else
         refresh(m_workingDirectory, &errorMessage);
 }
@@ -408,7 +410,7 @@ void BranchModel::renameTag(const QString &oldName, const QString &newName)
      || !m_client->synchronousTagCmd(m_workingDirectory,
                                      QStringList() << QLatin1String("-d") << oldName,
                                      &output, &errorMessage)) {
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
     } else {
         refresh(m_workingDirectory, &errorMessage);
     }
@@ -439,7 +441,7 @@ QString BranchModel::fullName(const QModelIndex &idx, bool includePrefix) const
     if (!node || !node->isLeaf())
         return QString();
     QStringList path = node->fullName(includePrefix);
-    return path.join(QString(QLatin1Char('/')));
+    return path.join(QLatin1Char('/'));
 }
 
 QStringList BranchModel::localBranchNames() const
@@ -498,7 +500,7 @@ void BranchModel::removeBranch(const QModelIndex &idx)
 
     args << QLatin1String("-D") << branch;
     if (!m_client->synchronousBranchCmd(m_workingDirectory, args, &output, &errorMessage)) {
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
         return;
     }
     removeNode(idx);
@@ -516,7 +518,7 @@ void BranchModel::removeTag(const QModelIndex &idx)
 
     args << QLatin1String("-d") << tag;
     if (!m_client->synchronousTagCmd(m_workingDirectory, args, &output, &errorMessage)) {
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
         return;
     }
     removeNode(idx);
@@ -545,7 +547,7 @@ bool BranchModel::branchIsMerged(const QModelIndex &idx)
 
     args << QLatin1String("-a") << QLatin1String("--contains") << sha(idx);
     if (!m_client->synchronousBranchCmd(m_workingDirectory, args, &output, &errorMessage))
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
 
     QStringList lines = output.split(QLatin1Char('\n'), QString::SkipEmptyParts);
     foreach (const QString &l, lines) {
@@ -590,7 +592,7 @@ QModelIndex BranchModel::addBranch(const QString &name, bool track, const QModel
     }
 
     if (!m_client->synchronousBranchCmd(m_workingDirectory, args, &output, &errorMessage)) {
-        VcsBase::VcsBaseOutputWindow::instance()->appendError(errorMessage);
+        VcsOutputWindow::appendError(errorMessage);
         return QModelIndex();
     }
 
