@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,20 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -35,59 +36,6 @@
 using namespace Core;
 using namespace Help::Internal;
 
-HelpFindSupport::HelpFindSupport(CentralWidget *centralWidget)
-    : m_centralWidget(centralWidget)
-{
-}
-
-HelpFindSupport::~HelpFindSupport()
-{
-}
-
-Core::FindFlags HelpFindSupport::supportedFindFlags() const
-{
-    return FindBackward | FindCaseSensitively;
-}
-
-QString HelpFindSupport::currentFindString() const
-{
-    QTC_ASSERT(m_centralWidget, return QString());
-    HelpViewer *viewer = m_centralWidget->currentHelpViewer();
-    if (!viewer)
-        return QString();
-    return viewer->selectedText();
-}
-
-QString HelpFindSupport::completedFindString() const
-{
-    return QString();
-}
-
-Core::IFindSupport::Result HelpFindSupport::findIncremental(const QString &txt,
-    Core::FindFlags findFlags)
-{
-    findFlags &= ~FindBackward;
-    return find(txt, findFlags, true) ? Found : NotFound;
-}
-
-Core::IFindSupport::Result HelpFindSupport::findStep(const QString &txt,
-    Core::FindFlags findFlags)
-{
-    return find(txt, findFlags, false) ? Found : NotFound;
-}
-
-bool HelpFindSupport::find(const QString &txt, Core::FindFlags findFlags, bool incremental)
-{
-    QTC_ASSERT(m_centralWidget, return false);
-    bool wrapped = false;
-    bool found = m_centralWidget->find(txt, findFlags, incremental, &wrapped);
-    if (wrapped)
-        showWrapIndicator(m_centralWidget);
-    return found;
-}
-
-// -- HelpViewerFindSupport
-
 HelpViewerFindSupport::HelpViewerFindSupport(HelpViewer *viewer)
     : m_viewer(viewer)
 {
