@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -110,19 +110,19 @@ static Abi findAbiOfMsvc(MsvcToolChain::Type type, MsvcToolChain::Platform platf
 
     switch (platform)
     {
-    case ProjectExplorer::Internal::MsvcToolChain::x86:
+    case MsvcToolChain::x86:
         wordWidth = 32;
         break;
-    case ProjectExplorer::Internal::MsvcToolChain::ia64:
-    case ProjectExplorer::Internal::MsvcToolChain::x86_ia64:
+    case MsvcToolChain::ia64:
+    case MsvcToolChain::x86_ia64:
         arch = Abi::ItaniumArchitecture;
         break;
-    case ProjectExplorer::Internal::MsvcToolChain::amd64:
-    case ProjectExplorer::Internal::MsvcToolChain::x86_amd64:
+    case MsvcToolChain::amd64:
+    case MsvcToolChain::x86_amd64:
         break;
-    case ProjectExplorer::Internal::MsvcToolChain::arm:
-    case ProjectExplorer::Internal::MsvcToolChain::x86_arm:
-    case ProjectExplorer::Internal::MsvcToolChain::amd64_arm:
+    case MsvcToolChain::arm:
+    case MsvcToolChain::x86_arm:
+    case MsvcToolChain::amd64_arm:
         arch = Abi::ArmArchitecture;
         wordWidth = 32;
         break;
@@ -239,27 +239,27 @@ QByteArray MsvcToolChain::msvcPredefinedMacros(const QStringList cxxflags,
     cpp.setEnvironment(env.toStringList());
     cpp.setWorkingDirectory(QDir::tempPath());
     QStringList arguments;
-    const QString binary = env.searchInPath(QLatin1String("cl.exe"));
+    const Utils::FileName binary = env.searchInPath(QLatin1String("cl.exe"));
     if (binary.isEmpty()) {
         qWarning("%s: The compiler binary cl.exe could not be found in the path.", Q_FUNC_INFO);
         return predefinedMacros;
     }
 
     arguments << toProcess << QLatin1String("/EP") << QDir::toNativeSeparators(saver.fileName());
-    cpp.start(binary, arguments);
+    cpp.start(binary.toString(), arguments);
     if (!cpp.waitForStarted()) {
-        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(binary),
+        qWarning("%s: Cannot start '%s': %s", Q_FUNC_INFO, qPrintable(binary.toUserOutput()),
             qPrintable(cpp.errorString()));
         return predefinedMacros;
     }
     cpp.closeWriteChannel();
     if (!cpp.waitForFinished()) {
         Utils::SynchronousProcess::stopProcess(cpp);
-        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(binary));
+        qWarning("%s: Timeout running '%s'.", Q_FUNC_INFO, qPrintable(binary.toUserOutput()));
         return predefinedMacros;
     }
     if (cpp.exitStatus() != QProcess::NormalExit) {
-        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(binary));
+        qWarning("%s: '%s' crashed.", Q_FUNC_INFO, qPrintable(binary.toUserOutput()));
         return predefinedMacros;
     }
 
@@ -308,7 +308,7 @@ static QString winExpandDelayedEnvReferences(QString in, const Utils::Environmen
 Utils::Environment MsvcToolChain::readEnvironmentSetting(Utils::Environment& env) const
 {
     Utils::Environment result = env;
-    if (!QFileInfo(m_vcvarsBat).exists())
+    if (!QFileInfo::exists(m_vcvarsBat))
         return result;
 
     QMap<QString, QString> envPairs;
@@ -354,7 +354,7 @@ bool MsvcToolChain::isValid() const
     if (!AbstractMsvcToolChain::isValid())
         return false;
     QString vcVarsBat = MsvcToolChainFactory::vcVarsBatFor(QFileInfo(m_vcvarsBat).absolutePath(), m_varsBatArg);
-    return QFileInfo(vcVarsBat).exists();
+    return QFileInfo::exists(vcVarsBat);
 }
 
 MsvcToolChain::MsvcToolChain() :
@@ -384,23 +384,26 @@ QString MsvcToolChain::typeDisplayName() const
 QList<Utils::FileName> MsvcToolChain::suggestedMkspecList() const
 {
     switch (m_abi.osFlavor()) {
-    case ProjectExplorer::Abi::WindowsMsvc2005Flavor:
+    case Abi::WindowsMsvc2005Flavor:
         return QList<Utils::FileName>() << Utils::FileName::fromLatin1("win32-msvc2005");
-    case ProjectExplorer::Abi::WindowsMsvc2008Flavor:
+    case Abi::WindowsMsvc2008Flavor:
         return QList<Utils::FileName>() << Utils::FileName::fromLatin1("win32-msvc2008");
-    case ProjectExplorer::Abi::WindowsMsvc2010Flavor:
+    case Abi::WindowsMsvc2010Flavor:
         return QList<Utils::FileName>() << Utils::FileName::fromLatin1("win32-msvc2010");
-    case ProjectExplorer::Abi::WindowsMsvc2012Flavor:
-        QList<Utils::FileName>()
+    case Abi::WindowsMsvc2012Flavor:
+        return QList<Utils::FileName>()
             << Utils::FileName::fromLatin1("win32-msvc2012")
             << Utils::FileName::fromLatin1("win32-msvc2010");
-        break;
-    case ProjectExplorer::Abi::WindowsMsvc2013Flavor:
-        QList<Utils::FileName>()
+    case Abi::WindowsMsvc2013Flavor:
+        return QList<Utils::FileName>()
             << Utils::FileName::fromLatin1("win32-msvc2013")
+            << Utils::FileName::fromLatin1("winphone-arm-msvc2013")
+            << Utils::FileName::fromLatin1("winphone-x86-msvc2013")
+            << Utils::FileName::fromLatin1("winrt-arm-msvc2013")
+            << Utils::FileName::fromLatin1("winrt-x86-msvc2013")
+            << Utils::FileName::fromLatin1("winrt-x64-msvc2013")
             << Utils::FileName::fromLatin1("win32-msvc2012")
             << Utils::FileName::fromLatin1("win32-msvc2010");
-        break;
     default:
         break;
     }
@@ -583,9 +586,12 @@ QList<ToolChain *> MsvcToolChainFactory::autoDetect()
         const QString vcvarsAllbat = path + QLatin1String("/vcvarsall.bat");
         if (QFileInfo(vcvarsAllbat).isFile()) {
             QList<MsvcToolChain::Platform> platforms; // prioritized list
+            // x86_arm was put before amd64_arm as a workaround for auto detected windows phone
+            // toolchains. As soon as windows phone builds support x64 cross builds, this change
+            // can be reverted.
             platforms << MsvcToolChain::x86
                       << MsvcToolChain::amd64 << MsvcToolChain::x86_amd64
-                      << MsvcToolChain::arm << MsvcToolChain::amd64_arm << MsvcToolChain::x86_arm
+                      << MsvcToolChain::arm << MsvcToolChain::x86_arm << MsvcToolChain::amd64_arm
                       << MsvcToolChain::ia64 << MsvcToolChain::x86_ia64;
             foreach (const MsvcToolChain::Platform &platform, platforms) {
                 if (hostSupportsPlatform(platform)
