@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -67,7 +67,7 @@ ItemLibraryWidget::ItemLibraryWidget(QWidget *parent) :
     m_itemIconSize(24, 24),
     m_resIconSize(24, 24),
     m_iconProvider(m_resIconSize),
-    m_itemsView(new QQuickView()),
+    m_itemViewQuickWidget(new QQuickWidget),
     m_resourcesView(new ItemLibraryTreeView(this)),
     m_filterFlag(QtBasic)
 {
@@ -76,16 +76,16 @@ ItemLibraryWidget::ItemLibraryWidget(QWidget *parent) :
     setWindowTitle(tr("Library", "Title of library view"));
 
     /* create Items view and its model */
-    m_itemsView->setResizeMode(QQuickView::SizeRootObjectToView);
+    m_itemViewQuickWidget->setResizeMode(QQuickWidget::SizeRootObjectToView);
     m_itemLibraryModel = new ItemLibraryModel(this);
 
-    QQmlContext *rootContext = m_itemsView->rootContext();
+    QQmlContext *rootContext = m_itemViewQuickWidget->rootContext();
     rootContext->setContextProperty(QStringLiteral("itemLibraryModel"), m_itemLibraryModel.data());
     rootContext->setContextProperty(QStringLiteral("itemLibraryIconWidth"), m_itemIconSize.width());
     rootContext->setContextProperty(QStringLiteral("itemLibraryIconHeight"), m_itemIconSize.height());
     rootContext->setContextProperty(QStringLiteral("rootView"), this);
 
-    m_itemsView->rootContext()->setContextProperty(QStringLiteral("highlightColor"), Utils::StyleHelper::notTooBrightHighlightColor());
+    m_itemViewQuickWidget->rootContext()->setContextProperty(QStringLiteral("highlightColor"), Utils::StyleHelper::notTooBrightHighlightColor());
 
     /* create Resources view and its model */
     m_resourcesFileSystemModel = new QFileSystemModel(this);
@@ -94,7 +94,7 @@ ItemLibraryWidget::ItemLibraryWidget(QWidget *parent) :
     m_resourcesView->setIconSize(m_resIconSize);
 
     /* create image provider for loading item icons */
-    m_itemsView->engine()->addImageProvider(QStringLiteral("qmldesigner_itemlibrary"), new Internal::ItemLibraryImageProvider);
+    m_itemViewQuickWidget->engine()->addImageProvider(QStringLiteral("qmldesigner_itemlibrary"), new Internal::ItemLibraryImageProvider);
 
     /* other widgets */
     QTabBar *tabBar = new QTabBar(this);
@@ -123,9 +123,9 @@ ItemLibraryWidget::ItemLibraryWidget(QWidget *parent) :
     lineEditLayout->addItem(new QSpacerItem(5, 5, QSizePolicy::Fixed, QSizePolicy::Fixed), 1, 2);
     connect(m_filterLineEdit.data(), SIGNAL(filterChanged(QString)), this, SLOT(setSearchFilter(QString)));
 
-    QWidget *container = createWindowContainer(m_itemsView.data());
+
     m_stackedWidget = new QStackedWidget(this);
-    m_stackedWidget->addWidget(container);
+    m_stackedWidget->addWidget(m_itemViewQuickWidget.data());
     m_stackedWidget->addWidget(m_resourcesView.data());
 
     QWidget *spacer = new QWidget(this);
@@ -229,7 +229,7 @@ void ItemLibraryWidget::setSearchFilter(const QString &searchFilter)
 {
     if (m_stackedWidget->currentIndex() == 0) {
         m_itemLibraryModel->setSearchText(searchFilter);
-        m_itemsView->update();
+        m_itemViewQuickWidget->update();
     } else {
         QStringList nameFilterList;
         if (searchFilter.contains('.')) {
@@ -300,8 +300,8 @@ void ItemLibraryWidget::reloadQmlSource()
 {
     QString itemLibraryQmlFilePath = qmlSourcesPath() + QStringLiteral("/ItemsView.qml");
     QTC_ASSERT(QFileInfo::exists(itemLibraryQmlFilePath), return);
-    m_itemsView->engine()->clearComponentCache();
-    m_itemsView->setSource(QUrl::fromLocalFile(itemLibraryQmlFilePath));
+    m_itemViewQuickWidget->engine()->clearComponentCache();
+    m_itemViewQuickWidget->setSource(QUrl::fromLocalFile(itemLibraryQmlFilePath));
 }
 
 void ItemLibraryWidget::setImportFilter(FilterChangeFlag flag)
@@ -374,10 +374,11 @@ void ItemLibraryWidget::setResourcePath(const QString &resourcePath)
     updateSearch();
 }
 
-static void ungrabMouseOnQMLWorldWorkAround(QQuickView *quickView)
+static void ungrabMouseOnQMLWorldWorkAround(QQuickWidget *quickWidget)
 {
-    if (quickView->mouseGrabberItem())
-        quickView->mouseGrabberItem()->ungrabMouse();
+    const QQuickWidgetPrivate *widgetPrivate = QQuickWidgetPrivate::get(quickWidget);
+    if (widgetPrivate && widgetPrivate->offscreenWindow && widgetPrivate->offscreenWindow->mouseGrabberItem())
+        widgetPrivate->offscreenWindow->mouseGrabberItem()->ungrabMouse();
 }
 
 void ItemLibraryWidget::startDragAndDrop(QVariant itemLibraryId)
@@ -392,7 +393,7 @@ void ItemLibraryWidget::startDragAndDrop(QVariant itemLibraryId)
 
     drag->exec();
 
-    ungrabMouseOnQMLWorldWorkAround(m_itemsView.data());
+    ungrabMouseOnQMLWorldWorkAround(m_itemViewQuickWidget.data());
 }
 
 void ItemLibraryWidget::removeImport(const QString &name)
