@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -158,6 +158,7 @@ struct SubmitEditorWidgetPrivate
     bool m_ignoreChange;
     bool m_descriptionMandatory;
     bool m_updateInProgress;
+    QString m_description;
 
     QActionPushButton *m_submitButton;
 };
@@ -177,8 +178,7 @@ SubmitEditorWidgetPrivate::SubmitEditorWidgetPrivate() :
 {
 }
 
-SubmitEditorWidget::SubmitEditorWidget(QWidget *parent) :
-    QWidget(parent),
+SubmitEditorWidget::SubmitEditorWidget() :
     d(new SubmitEditorWidgetPrivate)
 {
     d->m_ui.setupUi(this);
@@ -280,39 +280,48 @@ void SubmitEditorWidget::unregisterActions(QAction *editorUndoAction,  QAction *
 
 // Make sure we have one terminating NL. Do not trim front as leading space might be
 // required for some formattings.
-static inline QString trimMessageText(QString t)
+void SubmitEditorWidget::trimDescription()
 {
-    if (t.isEmpty())
-        return t;
+    if (d->m_description.isEmpty())
+        return;
     // Trim back of string.
-    const int last = t.size() - 1;
+    const int last = d->m_description.size() - 1;
     int lastWordCharacter = last;
-    for ( ; lastWordCharacter >= 0 && t.at(lastWordCharacter).isSpace() ; lastWordCharacter--) ;
+    for ( ; lastWordCharacter >= 0 && d->m_description.at(lastWordCharacter).isSpace() ; lastWordCharacter--) ;
     if (lastWordCharacter != last)
-        t.truncate(lastWordCharacter + 1);
-    t += QLatin1Char('\n');
-    return t;
+        d->m_description.truncate(lastWordCharacter + 1);
+    d->m_description += QLatin1Char('\n');
 }
 
 // Extract the wrapped text from a text edit, which performs
 // the wrapping only optically.
-static QString wrappedText(const QTextEdit *e)
+void SubmitEditorWidget::wrapDescription()
 {
+    if (!lineWrap())
+        return;
     const QChar newLine = QLatin1Char('\n');
-    QString rc;
-    QTextCursor cursor(e->document());
+    QTextEdit e;
+    e.setVisible(false);
+    e.setMinimumWidth(1000);
+    e.setFontPointSize(1.0);
+    e.setLineWrapColumnOrWidth(d->m_ui.description->lineWrapColumnOrWidth());
+    e.setLineWrapMode(d->m_ui.description->lineWrapMode());
+    e.setWordWrapMode(d->m_ui.description->wordWrapMode());
+    e.setPlainText(d->m_description);
+    d->m_description.clear();
+    QTextCursor cursor(e.document());
     cursor.movePosition(QTextCursor::Start);
     while (!cursor.atEnd()) {
         const QString block = cursor.block().text();
         if (block.startsWith(QLatin1Char('\t'))) { // Don't wrap
-            rc += block + newLine;
+            d->m_description += block + newLine;
             cursor.movePosition(QTextCursor::EndOfBlock);
         } else {
             forever {
                 cursor.select(QTextCursor::LineUnderCursor);
-                rc += cursor.selectedText();
-                rc += newLine;
-                cursor.movePosition(QTextCursor::EndOfLine); // Mac needs it
+                d->m_description += cursor.selectedText();
+                d->m_description += newLine;
+                cursor.clearSelection();
                 if (cursor.atBlockEnd())
                     break;
                 cursor.movePosition(QTextCursor::NextCharacter);
@@ -320,17 +329,11 @@ static QString wrappedText(const QTextEdit *e)
         }
         cursor.movePosition(QTextCursor::NextBlock);
     }
-    return rc;
 }
 
 QString SubmitEditorWidget::descriptionText() const
 {
-    QString rc = trimMessageText(lineWrap() ? wrappedText(d->m_ui.description) :
-                                              d->m_ui.description->toPlainText());
-    // append field entries
-    foreach (const SubmitFieldWidget *fw, d->m_fieldWidgets)
-        rc += fw->fieldValues();
-    return cleanupDescription(rc);
+    return d->m_description;
 }
 
 void SubmitEditorWidget::setDescriptionText(const QString &text)
@@ -557,14 +560,12 @@ void SubmitEditorWidget::hideDescription()
 
 void SubmitEditorWidget::descriptionTextChanged()
 {
-#if QT_VERSION < 0x050000 // Fix Qt-Bug, see QTCREATORBUG-5633 && QTCREATORBUG-6082
-    static QString lastText;
-    const QString text = d->m_ui.description->toPlainText();
-    if (lastText != text)
-        lastText = text;
-    else
-        return;
-#endif
+    d->m_description = cleanupDescription(d->m_ui.description->toPlainText());
+    wrapDescription();
+    trimDescription();
+    // append field entries
+    foreach (const SubmitFieldWidget *fw, d->m_fieldWidgets)
+        d->m_description += fw->fieldValues();
     updateSubmitAction();
 }
 
@@ -572,7 +573,7 @@ bool SubmitEditorWidget::canSubmit() const
 {
     if (d->m_updateInProgress)
         return false;
-    if (isDescriptionMandatory() && cleanupDescription(descriptionText()).trimmed().isEmpty())
+    if (isDescriptionMandatory() && d->m_description.trimmed().isEmpty())
         return false;
     const unsigned checkedCount = checkedFilesCount();
     return d->m_emptyFileListEnabled || checkedCount > 0;
