@@ -1,7 +1,7 @@
 /**************************************************************************
 **
-** Copyright (c) 2013 Brian McGillion and Hugues Delorme
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 Brian McGillion and Hugues Delorme
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,20 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -49,7 +50,7 @@ class ExitCodeInterpreter;
 
 namespace VcsBase {
 
-class Command;
+class VcsCommand;
 class VcsBaseEditorWidget;
 class VcsBaseClientSettings;
 class VcsJob;
@@ -92,7 +93,7 @@ public:
                                  const QString &dstLocation,
                                  const QStringList &extraOptions = QStringList());
     virtual void annotate(const QString &workingDir, const QString &file,
-                          const QString revision = QString(), int lineNumber = -1,
+                          const QString &revision = QString(), int lineNumber = -1,
                           const QStringList &extraOptions = QStringList());
     virtual void diff(const QString &workingDir, const QStringList &files = QStringList(),
                       const QStringList &extraOptions = QStringList());
@@ -131,7 +132,7 @@ public slots:
                       const QStringList &extraOptions = QStringList());
 
 protected:
-    enum VcsCommand
+    enum VcsCommandTag
     {
         CreateRepositoryCommand,
         CloneCommand,
@@ -149,9 +150,9 @@ protected:
         LogCommand,
         StatusCommand
     };
-    virtual QString vcsCommandString(VcsCommand cmd) const;
-    virtual Core::Id vcsEditorKind(VcsCommand cmd) const = 0;
-    virtual Utils::ExitCodeInterpreter *exitCodeInterpreter(VcsCommand cmd, QObject *parent) const;
+    virtual QString vcsCommandString(VcsCommandTag cmd) const;
+    virtual Core::Id vcsEditorKind(VcsCommandTag cmd) const = 0;
+    virtual Utils::ExitCodeInterpreter *exitCodeInterpreter(VcsCommandTag cmd, QObject *parent) const;
 
     virtual QStringList revisionSpec(const QString &revision) const = 0;
     virtual VcsBaseEditorParameterWidget *createDiffEditor(const QString &workingDir,
@@ -172,20 +173,21 @@ protected:
     Utils::SynchronousProcessResponse vcsSynchronousExec(const QString &workingDir,
                                                          const QStringList &args,
                                                          unsigned flags = 0,
-                                                         QTextCodec *outputCodec = 0);
-    VcsBase::VcsBaseEditorWidget *createVcsEditor(Core::Id kind, QString title,
-                                                  const QString &source, bool setSourceCodec,
-                                                  const char *registerDynamicProperty,
-                                                  const QString &dynamicPropertyValue) const;
+                                                         QTextCodec *outputCodec = 0) const;
+    VcsBaseEditorWidget *createVcsEditor(Core::Id kind, QString title,
+                                         const QString &source, bool setSourceCodec,
+                                         const char *registerDynamicProperty,
+                                         const QString &dynamicPropertyValue) const;
 
     enum JobOutputBindMode {
         NoOutputBind,
         VcsWindowOutputBind
     };
-    Command *createCommand(const QString &workingDirectory,
-                           VcsBase::VcsBaseEditorWidget *editor = 0,
-                           JobOutputBindMode mode = NoOutputBind);
-    void enqueueJob(Command *cmd, const QStringList &args, Utils::ExitCodeInterpreter *interpreter = 0);
+
+    VcsCommand *createCommand(const QString &workingDirectory,
+                           VcsBaseEditorWidget *editor = 0,
+                           JobOutputBindMode mode = NoOutputBind) const;
+    void enqueueJob(VcsCommand *cmd, const QStringList &args, Utils::ExitCodeInterpreter *interpreter = 0);
 
     void resetCachedVcsInfo(const QString &workingDir);
 
@@ -194,9 +196,9 @@ private:
     VcsBaseClientPrivate *d;
 
     Q_PRIVATE_SLOT(d, void statusParser(QString))
-    Q_PRIVATE_SLOT(d, void annotateRevision(QString, QString, QString, int))
+    Q_PRIVATE_SLOT(d, void annotateRevision(QString,QString,QString,int))
     Q_PRIVATE_SLOT(d, void saveSettings())
-    Q_PRIVATE_SLOT(d, void commandFinishedGotoLine(QWidget *))
+    Q_PRIVATE_SLOT(d, void commandFinishedGotoLine(QWidget*))
 };
 
 } //namespace VcsBase
