@@ -1,7 +1,7 @@
 #############################################################################
 ##
-## Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-## Contact: http://www.qt-project.org/legal
+## Copyright (C) 2015 The Qt Company Ltd.
+## Contact: http://www.qt.io/licensing
 ##
 ## This file is part of Qt Creator.
 ##
@@ -9,20 +9,21 @@
 ## Licensees holding valid commercial Qt licenses may use this file in
 ## accordance with the commercial license agreement provided with the
 ## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and Digia.  For licensing terms and
-## conditions see http://qt.digia.com/licensing.  For further information
-## use the contact form at http://qt.digia.com/contact-us.
+## a written agreement between you and The Qt Company.  For licensing terms and
+## conditions see http://www.qt.io/terms-conditions.  For further information
+## use the contact form at http://www.qt.io/contact-us.
 ##
 ## GNU Lesser General Public License Usage
 ## Alternatively, this file may be used under the terms of the GNU Lesser
-## General Public License version 2.1 as published by the Free Software
-## Foundation and appearing in the file LICENSE.LGPL included in the
-## packaging of this file.  Please review the following information to
-## ensure the GNU Lesser General Public License version 2.1 requirements
-## will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+## General Public License version 2.1 or version 3 as published by the Free
+## Software Foundation and appearing in the file LICENSE.LGPLv21 and
+## LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+## following information to ensure the GNU Lesser General Public License
+## requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+## http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 ##
-## In addition, as a special exception, Digia gives you certain additional
-## rights.  These rights are described in the Digia Qt LGPL Exception
+## In addition, as a special exception, The Qt Company gives you certain additional
+## rights.  These rights are described in The Qt Company LGPL Exception
 ## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 ##
 #############################################################################
@@ -115,7 +116,7 @@ def markText(editor, direction, typeCount=1):
 
 # works for all standard editors
 def replaceEditorContent(editor, newcontent):
-    type(editor, "<Ctrl+A>")
+    type(editor, "<Ctrl+a>")
     type(editor, "<Delete>")
     type(editor, newcontent)
 
@@ -152,9 +153,12 @@ def verifyHoveringOnEditor(editor, lines, additionalKeyPresses, expectedTypes, e
         for ty in additionalKeyPresses:
             type(editor, ty)
         rect = editor.cursorRect(editor.textCursor())
+        expectedToolTip = "{type='%s' visible='1'}" % expectedType
+        # wait for similar tooltips to disappear
+        checkIfObjectExists(expectedToolTip, False, 1000, True)
         sendEvent("QMouseEvent", editor, QEvent.MouseMove, rect.x+rect.width/2, rect.y+rect.height/2, Qt.NoButton, 0)
         try:
-            tip = waitForObject("{type='%s' visible='1'}" % expectedType)
+            tip = waitForObject(expectedToolTip)
         except:
             tip = None
         if tip == None:
@@ -222,7 +226,7 @@ def __handleColorTips__(colTip, expectedColor, alternativeColor):
     if cmp.alpha() != 255 or alternativeColor and alt.alpha() != 255:
         test.warning("Cannot handle transparent colors - cancelling this verification")
         return
-    dPM = QPixmap.grabWidget(colTip, 1, 1, colTip.width-2, colTip.height-2)
+    dPM = colTip.grab(QRect(1, 1, colTip.width - 2, colTip.height - 2))
     img = dPM.toImage()
     rgb = img.pixel(1, 1)
     rgb = QColor(rgb)
@@ -275,17 +279,12 @@ def getEditorForFileSuffix(curFile, treeViewSyntax=False):
             editor = waitForObject(":Qt Creator_CppEditor::Internal::CPPEditorWidget")
         elif suffix in qmlEditorSuffixes:
             editor = waitForObject(":Qt Creator_QmlJSEditor::QmlJSTextEditorWidget")
-        elif suffix in proEditorSuffixes:
-            editor = waitForObject(":Qt Creator_ProFileEditorWidget")
-        elif suffix in glslEditorSuffixes:
-            editor = waitForObject("{type='GLSLEditor::Internal::GLSLTextEditorWidget' unnamed='1' "
-                                   "visible='1' window=':Qt Creator_Core::Internal::MainWindow'}")
-        elif suffix in pytEditorSuffixes:
-            editor = waitForObject(":Qt Creator_PythonEditor::EditorWidget")
+        elif suffix in proEditorSuffixes or suffix in glslEditorSuffixes or suffix in pytEditorSuffixes:
+            editor = waitForObject(":Qt Creator_TextEditor::TextEditorWidget")
         else:
-            test.log("Trying PlainTextEditor (file suffix: %s)" % suffix)
+            test.log("Trying TextEditorWidget (file suffix: %s)" % suffix)
             try:
-                editor = waitForObject(":Qt Creator_TextEditor::PlainTextEditorWidget", 3000)
+                editor = waitForObject(":Qt Creator_TextEditor::TextEditorWidget", 3000)
             except:
                 test.fatal("Unsupported file suffix for file '%s'" % curFile)
                 editor = None
@@ -312,6 +311,13 @@ def maskSpecialCharsForSearchResult(filename):
     filename = filename.replace("_", "\\_").replace(".","\\.")
     return filename
 
+def waitForSearchResults():
+    cancelButton = ("{text='Cancel' type='QToolButton' unnamed='1' visible='1' "
+                    "window=':Qt Creator_Core::Internal::MainWindow'}")
+
+    waitFor("object.exists(cancelButton)", 3000)
+    waitFor("not object.exists(cancelButton)", 20000)
+
 def validateSearchResult(expectedCount):
     searchResult = waitForObject(":Qt Creator_SearchResult_Core::Internal::OutputPaneToggleButton")
     ensureChecked(searchResult)
